
import numpy as np
from matplotlib import pyplot as plt
import matplotlib.colors as mcolors
from mpl_toolkits.axes_grid1 import make_axes_locatable
import seaborn as sns
import copy
import cmocean
import matplotlib.patches as mpatches  
    
def state_by_elev(snow): 
    '''
    Plots SWE by elevation, delineated by melt/nonmelt
    
    '''
        
    lim = np.max(snow.melt[snow.total_lbl]) + np.max(snow.nonmelt[snow.total_lbl])
    ylim = np.max(lim) + np.max(lim)*0.3 
    colors = ['xkcd:rose red','xkcd:cool blue']
    fs = list(snow.figsize)
    fs[0] = fs[0]*0.9
    fs = tuple(fs)
    
    sns.set_style('darkgrid')
    sns.set_context("notebook")
    
    nf = len(snow.masks)
    
    if nf > 1 and nf < 5:
        nr = 2
        nc = 2
    elif nf < 7:
        nr = 2
        nc = 3
        
    if nf > 1:
        plt.close(7)
        fig,ax  = plt.subplots(num=7, figsize=fs, dpi=snow.dpi,
                               nrows = nr, ncols = nc)
        axs = ax.ravel()
        if nf == 5:
            fig.delaxes(axs[5])
               
        for iters,name in enumerate(snow.plotorder):
            # iters = 0
            # name = snow.plotorder[iters]
            axs[iters].bar(range(0,len(snow.edges)),snow.melt[name], 
                           color = colors[0], bottom = snow.nonmelt[name])
            axs[iters].bar(range(0,len(snow.edges)),snow.nonmelt[name], 
                           color = colors[1], label = 'unavail ')
            
            axs[iters].set_xlim(snow.xlims)                          
            xts = axs[iters].get_xticks()
            if len(xts) < 6:
                dxt = xts[1] - xts[0]
                xts = np.arange(xts[0],xts[-1] + 1 ,dxt/2)
                axs[iters].set_xticks(xts)                 
    
            edges_lbl = []
            for i in xts[0:len(xts)-1]:
                edges_lbl.append(str(int(snow.edges[int(i)])))
    
            axs[iters].set_xticklabels(str(i) for i in edges_lbl)
            
            if iters > nc - 1:
                if snow.units == 'KAF':
                    axs[iters].set_xlabel('elevation [ft]')
                if snow.units == 'SI':
                    axs[iters].set_xlabel('elevation [m]')                    
                  
            # Put yaxis on right 
            if iters == nc - 1 or iters == nf -1 :
                axs[iters].yaxis.set_label_position("right")
                axs[iters].yaxis.tick_right()
            
            if iters == 1 and nc == 3:
                axs[iters].set_yticklabels([])
                
            if iters <= nc - 1:
                axs[iters].set_xticklabels([])
            
            axs[iters].tick_params(axis='x')
            axs[iters].tick_params(axis='y')
            
            # Get basin total storage in strings for label
            kaf = str(np.int(sum(snow.melt[name])
                             + sum(snow.nonmelt[name]))) 
            
            if snow.dplcs == 0:
                kaf = str(np.int(sum(snow.melt[name])
                                 + sum(snow.nonmelt[name]))) 
            else: 
                kaf = str(np.round(sum(snow.melt[name])
                                   + sum(snow.nonmelt[name]),snow.dplcs))             
    
            if snow.units == 'KAF':          
                axs[iters].text(0.5,0.92,'%s - %s KAF'
                                %(snow.masks[name]['label'],kaf),
                                horizontalalignment='center',
                                transform=axs[iters].transAxes, fontsize = 10)
            if snow.units == 'SI':          
                axs[iters].text(0.5,0.92,'%s - %s $km^3$'
                                %(snow.masks[name]['label'],kaf),
                                horizontalalignment='center',
                                transform=axs[iters].transAxes)                
            
            if iters == 1 and nc == 3:
                axs[iters].set_yticklabels([])
            else:
                axs[iters].set_ylabel(snow.units)
            
            lbl = []
            for n in (0,1):
                if n == 0:
                    
                    if snow.dplcs == 0:
                        kafa = str(np.int(sum(snow.melt[name]))) 
                    else: 
                        kafa = str(np.round(sum(snow.melt[name]),snow.dplcs)) 
    
                    tmpa = (r'avail = %s')%(kafa)                         
                    lbl.append(tmpa)
    
                if snow.dplcs == 0:
                    kafna = str(np.int(sum(snow.nonmelt[name]))) 
                else: 
                    kafna = str(np.round(sum(snow.nonmelt[name]),snow.dplcs))            
    
                tmpna = ('unavail = %s')%(kafna)
                lbl.append(tmpna)                     
    
            axs[iters].legend(lbl, loc = (0.025, 0.65),fontsize = 9)
            
            if hasattr(snow,'elv_ylims'):
                axs[iters].set_ylim((snow.elv_ylims))
            else:
                axs[iters].set_ylim((0,ylim))          
            
            for tick in axs[iters].get_xticklabels():
                tick.set_rotation(30) 
        
        fig.tight_layout()
        for n in range(0,len(axs)):
            axs[n].set_xticks(xts)
            axs[n].set_xlim(snow.xlims)       
    
    else:
        name = snow.plotorder[0]
        plt.close(1)
        fig,ax  = plt.subplots(num=1, figsize=fs, dpi=snow.dpi)
               
        ax.bar(range(0,len(snow.edges)),snow.melt[name],
               color = colors[0], bottom = snow.nonmelt[name])
        ax.bar(range(0,len(snow.edges)),snow.nonmelt[name],
               color = colors[1], label = 'unavail ')
        
        ax.set_xlim(snow.xlims)                          
        xts = ax.get_xticks()
        edges_lbl = []
        for i in xts[0:len(xts)-1]:
            edges_lbl.append(str(int(snow.edges[int(i)])))
    
        ax.set_xticklabels(str(i) for i in edges_lbl)
    
        if snow.units == 'KAF':
            ax.set_xlabel('elevation [ft]')
        if snow.units == 'SI':
            ax.set_xlabel('elevation [m]')                    
        
        ax.tick_params(axis='x')
        ax.tick_params(axis='y')
        
        # Get basin total storage in strings for label
        kaf = str(np.int(sum(snow.melt[name]) + sum(snow.nonmelt[name]))) 
        
        if snow.dplcs == 0:
            kaf = str(np.int(sum(snow.melt[name]) 
                             + sum(snow.nonmelt[name]))) 
        else: 
            kaf = str(np.round(sum(snow.melt[name]) 
                               + sum(snow.nonmelt[name]),snow.dplcs))             
    
        if snow.units == 'KAF':          
            ax.text(.25,0.95,'%s - %s KAF'
                    %(snow.masks[name]['label'],kaf),
                    horizontalalignment='center',transform=ax.transAxes)
            ax.set_ylabel('KAF')
        if snow.units == 'SI':          
            ax.text(-0.88,2.1,'%s - %s $km^3$'
                    %(snow.masks[name]['label'],kaf),
                    horizontalalignment='center',transform=ax.transAxes)
            ax.set_ylabel(r'$km^3$')                
        
        lbl = []
        for n in (0,1):
            if n == 0:
                if snow.dplcs == 0:
                    kafa = str(np.int(sum(snow.melt[name]))) 
                else: 
                    kafa = str(np.round(sum(snow.melt[name]),snow.dplcs)) 
                
                tmpa = ('avail = %s')%(kafa)
                                      
                lbl.append(tmpa)
            
            # kafna = str(np.int(sum(snow.nonmelt[name]))) 
            if snow.dplcs == 0:
                kafna = str(np.int(sum(snow.nonmelt[name]))) 
            else: 
                kafna = str(np.round(sum(snow.nonmelt[name]),snow.dplcs))            
    
            tmpna = ('unavail = %s')%(kafna)                     
            lbl.append(tmpna)                     
    
        # axs[iters].legend(lbl,loc='upper left') 
        ax.legend(lbl, loc = (0.025, 0.8),fontsize = 9)
        ax.set_ylim((0,ylim)) 
        for tick in ax.get_xticklabels():
            tick.set_rotation(30)             
    
        fig.tight_layout()
        ax.set_xticks(xts)
        ax.set_xlim(snow.xlims)
    
    fig.subplots_adjust(top=0.92,wspace = 0.1)
    fig.suptitle('SWE, %s'%snow.dateTo.date().strftime("%Y-%-m-%-d"))
          
    if snow.elv_flag:
        print('saving figure to %sswe_elev%s.png'%(snow.figs_path,snow.name_append))   
        plt.savefig('%sswe_elev%s.png'%(snow.figs_path,snow.name_append)) 
         





import sys
import snowav

def run(config_file):

    # config_file = '/home/markrobertson/mrworkspace/code/SNOWAV/snowav/config/snowav_brb_wy2018.ini '
    # config_file = '/mnt/volumes/wkspace/config/snowav/snowav_brb_wy2018_devel.txt'
    snow = snowav.plotting.framework.SNOWAV(config_file)

    if not hasattr(snow,'error'):
        
        # Make all the calculations
        snow.process()
        
        # Save processed variables to netcdf if desired
        if snow.nc_flag == True:
            snowav.methods.output_nc.output_nc(snow)
    
        # Plots
        snowav.plotting.accumulated.accumulated(snow)
        snowav.plotting.current_image.current_image(snow)
        snowav.plotting.state_by_elev.state_by_elev(snow)
        snowav.plotting.image_change.image_change(snow)
        snowav.plotting.basin_total.basin_total(snow)
        snowav.plotting.pixel_swe.pixel_swe(snow)
        snowav.plotting.density.density(snow)
        snowav.plotting.water_balance.water_balance(snow)
        snowav.plotting.stn_validate.stn_validate(snow)
    
        # Generate report if desired
        if snow.report_flag == True:
            snowav.report.report.report(snow)

if __name__ == '__main__':
    config_file = sys.argv[1]
    run(config_file)




import numpy as np
from matplotlib import pyplot as plt
import matplotlib.colors as mcolors
from mpl_toolkits.axes_grid1 import make_axes_locatable
import seaborn as sns
import copy
import cmocean
import matplotlib.patches as mpatches


def pixel_swe(snow):
    
    
    plt.close(2) 
    fig,(ax,ax1) = plt.subplots(num=2, figsize=snow.figsize,
                                dpi=snow.dpi, nrows = 1, ncols = 2)

    sumorder = snow.plotorder[1:]
    if snow.basin == 'LAKES' or snow.basin == 'RCEW':
        sumorder = [snow.plotorder[0]]  
        swid = 0.45
    else:
        sumorder = snow.plotorder[1::]
        swid = 0.25
    
    wid = np.linspace(-0.3,0.3,len(sumorder))
    
    for iters,name in enumerate(sumorder): 
        # iters = 0
        # name = sumorder[iters]                             
        ax.bar(range(0,len(snow.edges))-wid[iters],
                snow.depth_mdep_byelev[name], 
                color = snow.barcolors[iters], width = swid, edgecolor = 'k',label = name) 
                    
        ax1.bar(range(0,len(snow.edges))-wid[iters],
                snow.state_mswe_byelev[name], 
                color = snow.barcolors[iters], width = swid, edgecolor = 'k',label = name)  
        
    ax.set_xlim((0,len(snow.edges)))    
    ax1.set_xlim((0,len(snow.edges)))        
    
    plt.tight_layout()                            
    xts         = ax1.get_xticks()
    edges_lbl   = []
    for i in xts[0:len(xts)-1]:
        edges_lbl.append(str(int(snow.edges[int(i)])))

    ax.set_xticklabels(str(i) for i in edges_lbl)
    ax1.set_xticklabels(str(i) for i in edges_lbl)
    for tick,tick1 in zip(ax.get_xticklabels(),ax1.get_xticklabels()):
        tick.set_rotation(30)
        tick1.set_rotation(30)  
         
    ylims = ax1.get_ylim()
    ax1.set_ylim((ylims[0],ylims[1]+ylims[1]*0.2))  
    
    ylims = ax.get_ylim()
    ax.set_ylim((ylims[0],ylims[1]+ylims[1]*0.2))               

    ax.set_ylabel('mean depth [%s]'%(snow.depthlbl))
    ax.set_xlabel('elevation [ft]')
    ax1.set_ylabel('mean SWE [%s]'%(snow.depthlbl))
    ax1.set_xlabel('elevation [ft]')
    ax.legend(loc='upper left')
    ax.grid('on')
    ax1.grid('on')
    
    ax1.yaxis.set_label_position("right")
    ax1.yaxis.tick_right()       
    
    ax.set_title('Mean Depth, %s'%(snow.dateTo.date().strftime("%Y-%-m-%-d")))
    ax1.set_title('Mean SWE, %s'%(snow.dateTo.date().strftime("%Y-%-m-%-d")))
    
    plt.tight_layout()
    print('saving figure to %smean_swe_depth%s.png'%(snow.figs_path,snow.name_append))
    plt.savefig('%smean_swe_depth%s.png'%(snow.figs_path,snow.name_append))   
    



import datetime

def wyhr_to_datetime(wy,wyhr):
    # wydt = wyhr_to_datetime(wy,wyhr)
    wyst        = datetime.datetime(wy-1,10,1,0,0,0)
    hrs         = datetime.timedelta(0,wyhr*60*60)
    wydatetime  = wyst + hrs
    
    return wydatetime



import numpy as np
from matplotlib import pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable
import seaborn as sns
import utm
import netCDF4 as nc
import mysql.connector
import pandas as pd
from datetime import datetime


def stn_validate(snow):
    
    if snow.valid_flag == False:
        print('No stations listed in config file for validation figure!')
        return
    
    rundirs = snow.run_dirs
    stns = snow.val_stns
    lbls = snow.val_lbls
    client = snow.val_client
       
    # get metadata from the data base from snotel sites
    if snow.basin == 'BRB':
        qry = ('SELECT tbl_metadata.* FROM tbl_metadata '
               + 'INNER JOIN tbl_stations ON tbl_metadata.primary_id = '
               + 'tbl_stations.station_id WHERE tbl_stations.client = '
               + ' "'"%s"'" HAVING network_name = "'"SNOTEL"'";'%client)
    else:
        qry = ('SELECT tbl_metadata.* FROM tbl_metadata ' 
               + 'INNER JOIN tbl_stations ON tbl_metadata.primary_id ='
               + 'tbl_stations.station_id WHERE tbl_stations.client = '
               + '"'"%s"'" ;'%client)        
    cnx = mysql.connector.connect(user='markrobertson', 
                                  password='whatdystm?1',
                                  host='10.200.28.137',
                                  database='weather_db')
    
    meta_sno = pd.read_sql(qry, cnx)
    meta_sno.index = meta_sno['primary_id']
    swe_meas    = pd.DataFrame(index = pd.date_range(datetime(2017,10,1), 
                                                     snow.dateTo, 
                                                     freq='D'),columns = stns)  
    swe_mod     = pd.DataFrame(index = pd.date_range(datetime(2017,10,1),
                                                     snow.dateTo, 
                                                     freq='D'),columns = stns)   
    tbl         = 'tbl_level1'
    var         = 'snow_water_equiv'
    st_time     = '2017-10-1 00:00:00'
    end_time    = snow.dateTo.date().strftime("%Y-%-m-%-d")
    
    # Get Snotel station results
    for iters,stn in enumerate(stns): 
        cnx     = mysql.connector.connect(user='markrobertson',
                                          password='whatdystm?1',
                                          host='10.200.28.137',
                                          port='32768',database='weather_db')
        var_qry = ('SELECT weather_db.%s.date_time, weather_db.%s.%s ' % (tbl,tbl,var) +
                    'FROM weather_db.%s ' % tbl +
                    "WHERE weather_db.%s.date_time between '" % tbl + st_time+ "' and '"+end_time+"'"
                    "AND weather_db.%s.station_id IN ('" % tbl + stn + "');")
        
        data = pd.read_sql(var_qry, cnx, index_col='date_time')
        dind = pd.date_range(st_time,end_time,freq='D')
        swe_meas[stn] = data.reindex(dind)
        
    if snow.basin == 'TUOL':
        swe_meas.TIOC1 = swe_meas.TIOC1 - 300  
        
    if 'AGP' in swe_meas:
        swe_meas.AGP = swe_meas.AGP - 40   
    if 'VLC' in swe_meas:
        swe_meas.VLC = swe_meas.VLC + 250  
    if 'UBC' in swe_meas:
        swe_meas.UBC = swe_meas.UBC - 50      
    
    sns.set_style('darkgrid')
    sns.set_context('notebook')
    
    plt.close(9)
    fig, axs = plt.subplots(num = 9,figsize = (10,10),nrows = 3,ncols = 2)   
    axs = axs.flatten() 
    
    ### sdwitching the loop!###
    
    # First need to combine all nc files... 
    px = (1,1,1,0,0,0,-1,-1,-1)
    py = (1,0,-1,1,0,-1,1,0,-1)       
    for iters,stn in enumerate(stns):
        # iters = 0
        # stn = stns[iters]
        for n,m in zip(px,py): 
            # n = 0
            # m = 0
            iswe = snow.offset
    
            for rname in rundirs:
                # rname = rundirs[1]
                
                ncpath  = rname[1].split('output')[0]
                ncf     = nc.Dataset(ncpath + 'snow.nc', 'r')    # open netcdf file
                nctvec  = ncf.variables['time'][:]
                vswe    = ncf.variables['specific_mass']            # get variable
                ncxvec  = ncf.variables['x'][:]                     # get x vec
                ncyvec  = ncf.variables['y'][:]                     # get y vec                      
                ll      = utm.from_latlon(meta_sno.ix[stn,'latitude'],meta_sno.ix[stn,'longitude']) # get utm coords from metadata
                # ll      = utm.from_latlon(37.641922,-119.055443)
                # ll      = utm.from_latlon(37.655201,-119.060783)
    
                xind    = np.where(abs(ncxvec-ll[0]) == min(abs(ncxvec-ll[0])))[0]  # get closest pixel index to the station
                yind    = np.where(abs(ncyvec-ll[1]) == min(abs(ncyvec-ll[1])))[0]  # get closest pixel index to the station
                # print(xind,yind)
                swe     = pd.Series(vswe[:,yind+m,xind+n].flatten(),index=nctvec)  # pull out closest model pixel data
         
                try:
                    swe_mod.loc[iswe:(iswe + len(swe.values)),stn] = swe.values  
                except:
                    sv = swe_mod[stn].values
                    lx = len(sv[iswe::])
                    swe_mod.loc[iswe:(iswe + lx),stn] = swe.values[0:lx]
                    
                ncf.close()   
                iswe = iswe + len(swe.values)
                
            z = snow.dem[yind,xind]    
           
            axs[iters].plot(swe_meas[stn],'k',label='measured')
            axs[iters].plot(swe_mod[stn],'b',linewidth = 0.75,label='model')    
            axs[iters].set_title(lbls[iters])
            axs[iters].set_xlim((datetime(2017, 10, 1),snow.dateTo))
        
        if iters == 1 or iters == 3 or iters == 5:
            axs[iters].yaxis.tick_right()
        
        if iters == 4 or iters == 5:
            for tick in axs[iters].get_xticklabels():
                tick.set_rotation(30) 
        else:
            axs[iters].set_xticklabels('')         
         
    # Plot
    maxm = np.nanmax(swe_meas)
    maxi = np.nanmax(swe_mod.max().values)
    
    if maxm > maxi:
        maxswe = maxm
    else:
        maxswe = maxi
        
    for iters in range(0,len(stns)):
        axs[iters].set_ylim((-0.1,maxswe + maxswe*0.05))     
    
    axs[0].legend(['measured','modelled'],loc='upper left')
    axs[0].set_ylabel('SWE [mm]')
    
    plt.suptitle('Validation at Measured Sites')
    plt.tight_layout()
    plt.subplots_adjust(top=0.92)
    
    snow.swe_meas = swe_meas
    snow.swe_mod = swe_mod
    
    print('saving figure to %svalidation%s.png'%(snow.figs_path,snow.name_append))   
    plt.savefig('%svalidation%s.png'%(snow.figs_path,snow.name_append))          


import netCDF4 as nc
import numpy as np


def output_nc(snow):
    '''
    Outputs attributes of snow identified in config file as netcdf file
    '''   
    for var in snow.ncvars:
        
        print('Saving %s to netcdf in %s%s%s'%(var,snow.figs_path,var,snow.name_append))
        p = getattr(snow,var)
        ny = len(p[:,0])
        nx = len(p[0,:])
        my_data = np.reshape(p,(nx,ny),'F')
        f = nc.Dataset('%s%s%s.nc'%(snow.figs_path,var,snow.name_append), 'w')
        f.createDimension('nx',nx)
        f.createDimension('ny',ny)
        data = f.createVariable(var, 'f4', ('nx','ny'))
        data[:] = np.flipud(my_data)
        f.close()  
    
        



from . import water_balance
from . import framework
from . import accumulated
from . import basin_total
from . import current_image
from . import density
from . import image_change
from . import pixel_swe
from . import framework
from . import state_by_elev
from . import stn_validate




# set the colormap and centre the colorbar
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as colors
from matplotlib.mlab import bivariate_normal

class MidpointNormalize(colors.Normalize):
    """
    Normalise the colorbar so that diverging bars work there way either side from a prescribed midpoint value)

    e.g. im=ax1.imshow(array, norm=MidpointNormalize(midpoint=0.,vmin=-100, vmax=100))
    """
    def __init__(self, vmin=None, vmax=None, midpoint=None, clip=False):
        self.midpoint = midpoint
        colors.Normalize.__init__(self, vmin, vmax, clip)

    def __call__(self, value, clip=None):
        # I'm ignoring masked values and all kinds of edge cases to make a
        # simple example...
        x, y = [self.vmin, self.midpoint, self.vmax], [0, 0.5, 1]
        return np.ma.masked_array(np.interp(value, x, y), np.isnan(value))



import numpy as np
from matplotlib import pyplot as plt
import matplotlib.colors as mcolors
from mpl_toolkits.axes_grid1 import make_axes_locatable
import seaborn as sns
import copy
import cmocean
import matplotlib.patches as mpatches


def water_balance(snow):
    
    cmap = plt.cm.PuBu
    
    precip = snow.precip
    
    sns.set_style('darkgrid')
    sns.set_context("notebook")

    plt.close(12)
    fig,(ax,ax1) = plt.subplots(num=12, 
                                figsize = snow.figsize, 
                                dpi=snow.dpi, 
                                nrows = 1, ncols = 2)      
    h = ax.imshow(precip,cmap = cmap)
    
    # Basin boundaries
    for name in snow.masks:
        ax.contour(snow.masks[name]['mask'],cmap = 'Greys',linewidths = 1)
    
    if snow.basin == 'SJ':
        fix1 = np.arange(1275,1377)
        fix2 = np.arange(1555,1618)
        ax.plot(fix1*0,fix1,'k')
        ax.plot(fix2*0,fix2,'k')
        
    if snow.basin == 'LAKES':
        ax.set_xlim(snow.imgx)
        ax.set_ylim(snow.imgy)
               
    # Do pretty stuff
    h.axes.get_xaxis().set_ticks([])
    h.axes.get_yaxis().set_ticks([])
    divider = make_axes_locatable(ax)
    
    cax = divider.append_axes("right", size="4%", pad=0.2)
    cbar = plt.colorbar(h, cax = cax)
    cbar.set_label('[%s]'%(snow.depthlbl))
    
    h.axes.set_title('Total Precipitation \n %s to %s'
                     %(snow.dateFrom.date().strftime("%Y-%-m-%-d"),
                       snow.dateTo.date().strftime("%Y-%-m-%-d")))  
          
    # Plot the bars
    # for iters,name in enumerate(snow.plotorder):
    iters = 0
    name = snow.plotorder[iters]
    
    ax1.plot(snow.precip_summary[name],label = 'precipitation') 
    
    ax1.plot(snow.accum_summary[name] 
             + snow.state_summary[name] 
             - snow.evap_summary[name],label = 'SWI + SWE - evap')
       
    ax1.legend()
    
    for tick in ax1.get_xticklabels():
        tick.set_rotation(30)    
        
    plt.tight_layout()  
    fig.subplots_adjust(top=0.88)  
       
    print('saving figure to %swater_balance%s.png'%(snow.figs_path,snow.name_append))   
    plt.savefig('%swater_balance%s.png'%(snow.figs_path,snow.name_append)) 
    
                  


from snowav.methods.MidpointNormalize import MidpointNormalize
import numpy as np
from matplotlib import pyplot as plt
import matplotlib.colors as mcolors
from mpl_toolkits.axes_grid1 import make_axes_locatable
import seaborn as sns
import copy
import cmocean
import matplotlib.patches as mpatches
import colorcet as cc


def density(snow): 
    '''
    None of these are really finished...
    '''
    
    value = copy.deepcopy(snow.density_m_byelev)
    lim = np.max(value[snow.total_lbl]) 
    ylim = (0,600) 
    color = 'xkcd:windows blue'

    sns.set_style('darkgrid')
    sns.set_context("notebook")        

    nf = len(snow.masks)
    
    if nf > 1 and nf < 5:
        nr = 2
        nc = 2
    elif nf < 7:
        nr = 2
        nc = 3
        
    plt.close(3)
    fig,ax  = plt.subplots(num=3, figsize=snow.figsize, dpi=snow.dpi, nrows = nr, ncols = nc)
    axs     = ax.ravel()
    if nf == 5:
        fig.delaxes(axs[5])
           
    for iters,name in enumerate(snow.plotorder):
        # iters = 0
        # name = snow.plotorder[iters]
        axs[iters].bar(range(0,len(snow.edges)),value[name], color = color)
        
        axs[iters].set_xlim(snow.xlims)                          
        xts         = axs[iters].get_xticks()
        if len(xts) < 6:
            dxt = xts[1] - xts[0]
            xts = np.arange(xts[0],xts[-1] + 1 ,dxt/2)
            axs[iters].set_xticks(xts)                 

        edges_lbl   = []
        for i in xts[0:len(xts)-1]:
            edges_lbl.append(str(int(snow.edges[int(i)])))

        axs[iters].set_xticklabels(str(i) for i in edges_lbl)
        
        if iters == 0:
            axs[iters].set_ylabel(r'$\rho$ [kg/$m^3$]') 
        
        if iters > nc - 1:
            axs[iters].set_xlabel('elevation [%s]'%(snow.elevlbl))

        # Put yaxis on right 
        if iters == nc - 1 or iters == nf -1 :
            axs[iters].yaxis.set_label_position("right")
            axs[iters].yaxis.tick_right()
            axs[iters].set_ylabel(r'$\rho$ [kg/$m^3$]') 
        
        if iters == 1 and nc == 3:
            axs[iters].set_yticklabels([])
            
        if iters <= nc - 1:
            axs[iters].set_xticklabels([])
        
        axs[iters].set_ylim((ylim)) 
        for tick in axs[iters].get_xticklabels():
            tick.set_rotation(30) 
            
        axs[iters].text(0.5,0.92,name,horizontalalignment='center',transform=axs[iters].transAxes,fontsize = 10)
    
    for n in range(0,len(axs)):
        axs[n].set_xticks(xts)
        axs[n].set_xlim(snow.xlims)       

    fig.tight_layout()
    fig.subplots_adjust(top=0.92,wspace = 0.1)
    fig.suptitle(r'Density, %s'%(snow.dateTo.date().strftime("%Y-%-m-%-d")) )       
    
    print('saving figure to %sdensity_subs%s.png'%(snow.figs_path,snow.name_append))
    plt.savefig('%sdensity_subs%s.png'%(snow.figs_path,snow.name_append))     
    
    ###############################
    # 2nd density fig
    ###############################

    cvalue = copy.deepcopy(snow.density)
    
    colors1 = cmocean.cm.speed(np.linspace(0., 1, 255))
    colors2 = plt.cm.binary(np.linspace(0, 1, 1))
    colors = np.vstack((colors2, colors1))
    mymap = mcolors.LinearSegmentedColormap.from_list('my_colormap', colors)

    sns.set_style('darkgrid')
    sns.set_context("notebook")
    
    # This is to get the background white
    pmask = snow.masks[snow.total_lbl]['mask']
    ixo = pmask == 0
    cvalue[ixo] = np.nan
    mymap.set_bad('white',1.) 
    
    ixf = cvalue == 0
    cvalue[ixf] = -1
    mymap.set_under('lightgrey',1.) 
    
    plt.close(4)
    fig,(ax,ax1) = plt.subplots(num=4, figsize = snow.figsize, 
                                dpi=snow.dpi, nrows = 1, ncols = 2)      
    h = ax.imshow(cvalue, clim = (50,550), interpolation='none', cmap = mymap)
    
    # Basin boundaries
    for name in snow.masks:
        ax.contour(snow.masks[name]['mask'],cmap = 'Greys',linewidths = 1)
    
    if snow.basin == 'SJ':
        fix1 = np.arange(1275,1377)
        fix2 = np.arange(1555,1618)
        ax.plot(fix1*0,fix1,'k')
        ax.plot(fix2*0,fix2,'k')
        
    if snow.basin == 'LAKES':
        ax.set_xlim(snow.imgx)
        ax.set_ylim(snow.imgy)
               
    # Do pretty stuff
    h.axes.get_xaxis().set_ticks([])
    h.axes.get_yaxis().set_ticks([])
    divider = make_axes_locatable(ax)
    
    cax = divider.append_axes("right", size="4%", pad=0.2)
    cbar = plt.colorbar(h, cax = cax)
    cbar.ax.tick_params() 
    cbar.set_label('[kg/$m^3$]')

    h.axes.set_title('Density\n%s'%(snow.dateTo.date().strftime("%Y-%-m-%-d")))  

    ax1.bar(range(0,len(snow.edges)),value[snow.total_lbl], 
            color = 'g', edgecolor = 'k')
  
    plt.rcParams['hatch.linewidth'] = 1
    plt.rcParams['hatch.color'] = 'k'                
    ax1.set_xlim((snow.xlims[0]-0.5,snow.xlims[1]))
    
    plt.tight_layout()
    xts = ax1.get_xticks()
    edges_lbl = []
    for i in xts[0:len(xts)-1]:
        edges_lbl.append(str(int(snow.edges[int(i)])))
    
    ax1.set_xticklabels(str(i) for i in edges_lbl)
    for tick in ax1.get_xticklabels():
        tick.set_rotation(30)        

    ax1.set_ylabel('density - per elevation band')
    ax1.set_xlabel('elevation [%s]'%(snow.elevlbl))
   
    ax1.yaxis.set_label_position("right")
    ax1.yaxis.tick_right()
    ax1.set_ylim((0,600))

    plt.tight_layout()
    fig.subplots_adjust(top=0.88)

    if sum(sum(ixf)) > 1000:
        patches = [mpatches.Patch(color='grey', label='snow free')]
        if snow.basin == 'SJ':
            ax.legend(handles=patches, bbox_to_anchor=(0.3, 0.05), loc=2, borderaxespad=0. )
        else:
            ax.legend(handles=patches, bbox_to_anchor=(0.05, 0.05), loc=2, borderaxespad=0. )
    
    print('saving figure to %sdensity%s.png'%(snow.figs_path,snow.name_append))
    plt.savefig('%sdensity%s.png'%(snow.figs_path,snow.name_append))         
    
    
    ###############################
    # 3rd density fig
    ###############################
    
    nsub = 100
    depth  = copy.deepcopy(snow.depth)
    density = cvalue
    swe = copy.deepcopy(snow.state)
    
    depths = np.reshape(depth,(1,len(depth[:,0])*len(depth[0,:])))
    densitys = np.reshape(density,(1,len(depth[:,0])*len(density[0,:])))
    swesats = np.reshape(swe,(1,len(swe[:,0])*len(swe[0,:])))
    densitys = densitys[0,:]
    depths = depths[0,:]
    swesats = swesats[0,:]
    depthsub = depths[::nsub]
    densitysub = densitys[::nsub]
    swesub = swesats[::nsub]
    
    z = snow.dem
    zs = np.reshape(z,(1,len(z[:,0])*len(z[0,:])))
    zs = zs[0,:]
    zsub = zs[::100]
    
    cm = cc.m_bgy
    
    plt.close(20)
    fig = plt.figure(num=20, figsize = (6,4), dpi=snow.dpi)      
    ax = plt.gca()       
    h = ax.scatter(swesub,densitysub,c=zsub,vmin=5000,vmax=12500 ,cmap=cm,s=5)
    
    ax.set_ylabel(r'$\rho$ [kg/$m^3$]')
    ax.set_xlabel('SWE [in]')
    
    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="4%", pad=0.2)
    cbar = plt.colorbar(h, cax = cax)
    cbar.ax.tick_params() 
    cbar.set_label('elevation [ft]')
    plt.tight_layout()
          
    print('saving figure to %sdensity_swe%s.png'%(snow.figs_path,snow.name_append))
    plt.savefig('%sdensity_swe%s.png'%(snow.figs_path,snow.name_append))          




import numpy as np
from matplotlib import pyplot as plt
import matplotlib.colors as mcolors
from mpl_toolkits.axes_grid1 import make_axes_locatable
import seaborn as sns
import copy
import cmocean
import matplotlib.patches as mpatches

def current_image(snow):
    '''
    This plots snow.state (typically SWE) and snow.cold
    
    edits: make flexible input arguments for depth, density, etc
    
    '''
    # Make a copy so we can edit for plots

    state = copy.deepcopy(snow.state)
    cold = copy.deepcopy(snow.cold)

    qMin,qMax = np.nanpercentile(state,[0,99.9])    
    clims = (qMin,qMax)
    clims2 = (-5,0) 
    
    # Areas outside basin
    pmask = snow.masks[snow.total_lbl]['mask']
    ixo = pmask == 0   
    
    # Prepare no-snow and outside of the basin for the colormaps   
    ixz = state == 0
    state[ixz] = -1
    cold[ixz]  = 1     
    
    # Colormap for snow.state
    colorsbad = plt.cm.Set2_r(np.linspace(0., 1, 1))
    colors1 = cmocean.cm.haline_r(np.linspace(0., 1, 254))
    colors = np.vstack((colorsbad,colors1))
    mymap = mcolors.LinearSegmentedColormap.from_list('my_colormap', colors) 
    
    state[ixo] = np.nan        
    mymap.set_bad('white',1.) 
    # mymap.set_under('lightgrey',1)    
    
    # Colormap for cold content
    # colorsbad       = plt.cm.binary(np.linspace(0., 1, 1))
    # colors1         = plt.cm.ocean_r(np.linspace(0., 1, 128))
    # colors2         = plt.cm.YlOrRd(np.linspace(0., 1, 127))
    # colors          = np.vstack((colorsbad, colors1, colors2))
    # mymap1          = mcolors.LinearSegmentedColormap.from_list('my_colormap', colors)
    # mymap1          = plt.cm.RdYlBu_r 
    # mymap1          = cc.m_diverging_linear_bjy_30_90_c45
    mymap1 = plt.cm.Spectral_r
    cold[ixo] = np.nan
    mymap1.set_bad('white')
    mymap1.set_over('lightgrey',1) 
                     
    sns.set_style('dark')
    sns.set_context("notebook")
    
    plt.close(1)
    fig,(ax,ax1) = plt.subplots(num=1, figsize=snow.figsize, 
                                facecolor = 'white', dpi=snow.dpi, 
                                nrows = 1, ncols = 2)
    h = ax.imshow(state, cmap = mymap, clim=clims)
    h1 = ax1.imshow(cold, clim=clims2, cmap = mymap1)
    
    if snow.basin == 'LAKES':
        ax.set_xlim(snow.imgx)
        ax.set_ylim(snow.imgy)
        ax1.set_xlim(snow.imgx)
        ax1.set_ylim(snow.imgy)
    
    # Basin boundaries
    for name in snow.masks:
        ax.contour(snow.masks[name]['mask'],cmap = "Greys",linewidths = 1)
        ax1.contour(snow.masks[name]['mask'],cmap = "Greys",linewidths = 1)
        
    if snow.basin == 'SJ':
        fix1 = np.arange(1275,1377)
        fix2 = np.arange(1555,1618)
        ax.plot(fix1*0,fix1,'k')
        ax.plot(fix2*0,fix2,'k')
        ax1.plot(fix1*0,fix1,'k')
        ax1.plot(fix2*0,fix2,'k')
    
    # Do pretty stuff for the left plot
    h.axes.get_xaxis().set_ticks([])
    h.axes.get_yaxis().set_ticks([])
    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="5%", pad=0.2)
    cbar = plt.colorbar(h, cax = cax)
    
    if snow.units == 'KAF':         
        cbar.set_label('[in]')
    if snow.units == 'SI':
        cbar.set_label('[mm]')
     
    # cbar.ax.tick_params()                
    
    # Do pretty stuff for the right plot
    h1.axes.get_xaxis().set_ticks([])
    h1.axes.get_yaxis().set_ticks([])
    h1.axes.set_title('Cold Content \n %s'%(snow.dateTo.date().strftime("%Y-%-m-%-d")))
    divider = make_axes_locatable(ax1)
    cax2 = divider.append_axes("right", size="5%", pad=0.2)
    cbar1 = plt.colorbar(h1, cax = cax2)
    cbar1.set_label('Cold Content [MJ/$m^3$]')
    cbar1.ax.tick_params() 
    
    h.axes.set_title('SWE \n %s'%(snow.dateTo.date().strftime("%Y-%-m-%-d")))
    fig.subplots_adjust(top=0.95,bottom=0.05,
                        right = 0.92, left = 0.05, wspace = 0.12)
    if snow.basin == 'LAKES':
        plt.tight_layout()
    
    patches = [mpatches.Patch(color='grey', label='snow free')]
    if snow.basin == 'SJ':
        ax.legend(handles=patches, bbox_to_anchor=(0.3, 0.05), loc=2, borderaxespad=0. )
    if snow.basin == 'RCEW':
        ax.legend(handles=patches, bbox_to_anchor=(-0.2, 0.05), loc=2, borderaxespad=0. )            
    else:
        ax.legend(handles=patches, bbox_to_anchor=(0.05, 0.05), loc=2, borderaxespad=0. )
    
    print('saving figure to %sresults%s.png'%(snow.figs_path,snow.name_append))
    plt.savefig('%sresults%s.png'%(snow.figs_path,snow.name_append))  
    



import numpy as np
from matplotlib import pyplot as plt
import matplotlib.colors as mcolors
from mpl_toolkits.axes_grid1 import make_axes_locatable
import seaborn as sns
import copy
import cmocean
import matplotlib.patches as mpatches


def accumulated(snow):
    '''
    Issues:
    - matplotlib, multiple colormaps, and qMin/qMax do not play nice
    
    '''

    if snow.acc_flag == 'False':
        print('Config option, not creating accumulated() fig')
        return
         
    # Only report accum by the subsection between 
    accum = copy.deepcopy(snow.accum_sub) 
    accum_byelev = copy.deepcopy(snow.accum_byelev_sub) 
    
    qMin,qMax = np.percentile(accum,[0,99.8])
    clims = (0,qMax)
    colors1 = cmocean.cm.dense(np.linspace(0., 1, 255))
    colors2 = plt.cm.binary(np.linspace(0, 1, 1))
    colors = np.vstack((colors2, colors1))
    mymap = mcolors.LinearSegmentedColormap.from_list('my_colormap', colors)
     
    sns.set_style('darkgrid')
    sns.set_context("notebook")
    
    # White background
    pmask = snow.masks[snow.total_lbl]['mask']
    ixo = pmask == 0
    accum[ixo] = np.nan
    mymap.set_bad('white',1.) 
    
    # Now set SWI-free to some color

    ixf = accum == 0
    accum[ixf] = -1
    mymap.set_under('grey',1.) 
    
    plt.close(0)
    fig,(ax,ax1) = plt.subplots(num=0, 
                                figsize = snow.figsize, 
                                dpi=snow.dpi, 
                                nrows = 1, ncols = 2)      
    h = ax.imshow(accum, clim = clims, cmap = mymap)
    
    # Basin boundaries
    for name in snow.masks:
        ax.contour(snow.masks[name]['mask'],cmap = 'Greys',linewidths = 1)
    
    if snow.basin == 'SJ':
        fix1 = np.arange(1275,1377)
        fix2 = np.arange(1555,1618)
        ax.plot(fix1*0,fix1,'k')
        ax.plot(fix2*0,fix2,'k')
        
    if snow.basin == 'LAKES':
        ax.set_xlim(snow.imgx)
        ax.set_ylim(snow.imgy)
               
    # Do pretty stuff
    h.axes.get_xaxis().set_ticks([])
    h.axes.get_yaxis().set_ticks([])
    divider = make_axes_locatable(ax)
    
    cax = divider.append_axes("right", size="4%", pad=0.2)
    cbar = plt.colorbar(h, cax = cax)
    # cbar.ax.tick_params() 
    cbar.set_label('[%s]'%(snow.depthlbl))
    
    h.axes.set_title('Accumulated SWI \n %s to %s'
                     %(snow.dateFrom.date().strftime("%Y-%-m-%-d"),
                       snow.dateTo.date().strftime("%Y-%-m-%-d")))  
    
    # Total basin label
    sumorder = snow.plotorder[1:]
    if snow.basin == 'LAKES' or snow.basin == 'RCEW':
        sumorder = [snow.plotorder[0]]
        
    if snow.dplcs == 0:
        tlbl = '%s = %s %s'%(snow.plotorder[0],
                             str(int(accum_byelev[snow.plotorder[0]].sum())),
                             snow.vollbl)
    else: 
        tlbl = '%s = %s %s'%(snow.plotorder[0],
                             str(np.round(accum_byelev[snow.plotorder[0]].sum(),
                            snow.dplcs)),snow.vollbl)
          
    # Plot the bars
    for iters,name in enumerate(sumorder):
        if snow.dplcs == 0:           
            lbl = '%s = %s %s'%(name,str(int(accum_byelev[name].sum())),
                                snow.vollbl)
        else:
            lbl = '%s = %s %s'%(name,str(np.round(accum_byelev[name].sum(),
                                snow.dplcs)),snow.vollbl)
    
        if iters == 0:
            ax1.bar(range(0,len(snow.edges)),accum_byelev[name], 
                    color = snow.barcolors[iters], 
                    edgecolor = 'k',label = lbl)
        elif iters == 1:   
            ax1.bar(range(0,len(snow.edges)),accum_byelev[name], 
                    bottom = accum_byelev[sumorder[iters-1]], 
                    color = snow.barcolors[iters], edgecolor = 'k',label = lbl)
          
        elif iters == 2:   
            ax1.bar(range(0,len(snow.edges)),accum_byelev[name], 
                    bottom = (accum_byelev[sumorder[iters-1]] + accum_byelev[sumorder[iters-2]]), 
                    color = snow.barcolors[iters], edgecolor = 'k',label = lbl)
    
        elif iters == 3:   
            ax1.bar(range(0,len(snow.edges)),accum_byelev[name], 
                    bottom = (accum_byelev[sumorder[iters-1]] + accum_byelev[sumorder[iters-2]] + accum_byelev[sumorder[iters-3]]), 
                    color = snow.barcolors[iters], edgecolor = 'k',label = lbl)
            
           
        plt.rcParams['hatch.linewidth'] = 1
        plt.rcParams['hatch.color'] = 'k'                
        ax1.set_xlim((snow.xlims[0]-0.5,snow.xlims[1]))
    
    plt.tight_layout()
    xts         = ax1.get_xticks()
    edges_lbl   = []
    for i in xts[0:len(xts)-1]:
        edges_lbl.append(str(int(snow.edges[int(i)])))
    
    ax1.set_xticklabels(str(i) for i in edges_lbl)
    for tick in ax1.get_xticklabels():
        tick.set_rotation(30)        
    
    ax1.set_ylabel('%s - per elevation band'%(snow.vollbl))
    ax1.set_xlabel('elevation [%s]'%(snow.elevlbl))
    
    ax1.yaxis.set_label_position("right")
    ax1.yaxis.tick_right()
    
    if hasattr(snow,'acc_ylims'):
        ax1.set_ylim((snow.acc_ylims))
    else:
        ylims = ax1.get_ylim()
        ax1.set_ylim((0,ylims[1] + ylims[1]*0.2))
    
    plt.tight_layout()
    fig.subplots_adjust(top=0.88)
    
    if snow.basin != 'LAKES' and snow.basin != 'RCEW':
        # more ifs for number subs...
        if len(snow.plotorder) == 5:
            ax1.legend(loc= (0.01,0.68))
        elif len(snow.plotorder) == 4:
            ax1.legend(loc= (0.01,0.74))
        
    if snow.basin == 'BRB':
        ax1.text(0.26,0.94,tlbl,horizontalalignment='center',
                 transform=ax1.transAxes,fontsize = 10)
    else:
        ax1.text(0.3,0.94,tlbl,horizontalalignment='center',
                 transform=ax1.transAxes,fontsize = 10)
    
    # Make SWI-free legend if we need one
    if sum(sum(ixf)) > 1000:
        patches = [mpatches.Patch(color='grey', label='no SWI')]
        if snow.basin == 'SJ':
            ax.legend(handles=patches, bbox_to_anchor=(0.3, 0.05), 
                      loc=2, borderaxespad=0. )
        else:
            ax.legend(handles=patches, bbox_to_anchor=(0.05, 0.05), 
                      loc=2, borderaxespad=0. )
         
    # Save if true
    if snow.acc_flag:
        print('saving figure to %sswi%s.png'%(snow.figs_path,snow.name_append))
        plt.savefig('%sswi%s.png'%(snow.figs_path,snow.name_append))  
    
     


from jinja2 import FileSystemLoader
from latex.jinja2 import make_env
from latex import build_pdf
import pandas as pd
import numpy as np
from datetime import datetime
import copy


def report(obj):
    '''
    
    '''
    
    # Initialize all the variables to pass to latex file
    variables = {}
       
    variables['TOTAL_SWI'] = obj.accum_byelev[obj.total_lbl].sum()
    variables['TOTAL_SWE'] = obj.state_byelev[obj.total_lbl].sum()
    variables['TOTAL_SWE_AV'] = obj.melt[obj.total_lbl].sum()
    variables['TOTAL_SWEDEL'] = obj.delta_state_byelev[obj.total_lbl].sum()
    variables['TOTAL_PM'] = (np.nansum(
                            np.multiply(obj.state 
                            * obj.masks[obj.total_lbl]['mask'],1))
                            / obj.masks[obj.total_lbl]['mask'].sum())
    variables['TOTALPRE_PM'] = (np.nansum(
                                np.multiply(obj.precip 
                                * obj.masks[obj.total_lbl]['mask'],1))
                                / obj.masks[obj.total_lbl]['mask'].sum() )
    total_rai = obj.rain_bg_byelev[obj.total_lbl].sum()
    variables['TOTAL_RAT'] = str(int((total_rai 
                                      /obj.accum_byelev[obj.total_lbl].sum())*100))
  
    report_time = datetime.now().strftime("%Y-%-m-%-d %H:%M")   
    if hasattr(obj,'orig_date'):
        report_time = obj.orig_date + ', revised ' + report_time
   
    # HACK, somewhere the rounding is different when we start from Oct 1...
    # if obj.dateFrom == datetime(2017,10,1,23,0):
    #     total_swe_del = copy.copy(total_swe)
    
    numsubs = range(1,len(obj.plotorder))

    for n,sub in zip(numsubs,obj.suborder):
        SWIIND = 'SUB' + str(n) + '_SWI'
        SWEIND = 'SUB' + str(n) + '_SWE'
        AVSWEIND = 'SUB' + str(n) + '_SWE_AV'
        SWEDEL = 'SUB' + str(n) + '_SWEDEL'
        PM = 'SUB' + str(n) + '_PM'
        PREPM = 'SUB' + str(n) + 'PRE_PM'
        RAIN = 'SUB' + str(n) + 'RAI'
        RATIO = 'SUB' + str(n) + '_RAT'
        
        swival = obj.accum_byelev[sub].sum()
        sweval = obj.state_byelev[sub].sum() 
        avsweval = obj.melt[sub].sum()
        swedelval = obj.delta_state_byelev[sub].sum()
        pmval = (np.nansum(np.multiply(obj.state
                                      * obj.masks[sub]['mask'],1))
                                      / obj.masks[sub]['mask'].sum())
        prepmval = (np.nansum(np.multiply(obj.precip*obj.masks[sub]['mask'],1))
                        /obj.masks[sub]['mask'].sum())
        rainval = obj.rain_bg_byelev[sub].sum()
        ratval = str(int((rainval/swival)*100))
        
        variables[SWIIND] = swival
        variables[SWEIND] = sweval
        variables[AVSWEIND] = avsweval
        variables[SWEDEL] = swedelval
        variables[PM] = pmval
        variables[PREPM] = prepmval
        variables[RAIN] = rainval
        variables[RATIO]  = ratval

    # Assign some more variables
    start_date = obj.dateFrom.date().strftime("%B %-d")
    end_date = obj.dateTo.date().strftime("%B %-d")  
    
    report_title = obj.rep_title
    fore_date = ' '
    swe_in = obj.state_byelev[obj.total_lbl].sum()
    swi_in = obj.accum_byelev[obj.total_lbl].sum()
    vollbl = obj.vollbl
    deplbl = obj.depthlbl        
    
    fig_path = obj.figs_path
    swi_fig = 'swi%s.png'%(obj.name_append)
    results_fig = 'results%s.png'%(obj.name_append)
    changes_fig = 'swe_change%s.png'%(obj.name_append)
    elev_fig = 'swe_elev%s.png'%(obj.name_append)
    totals_fig = 'basin_total%s.png'%(obj.name_append)
    totalsmy_fig = 'basin_total_multiyr%s.png'%(obj.name_append)
    valid_fig = 'validation%s.png'%(obj.name_append)
    hyp_fig  = 'hypsometry%s.png'%(obj.name_append)
    mean_fig = 'mean_swe_depth%s.png'%(obj.name_append)
    detail_fig = 'mean_detail%s.png'%(obj.name_append)
    density_fig = 'density%s.png'%(obj.name_append)
    density_sub_fig = 'density_sub%s.png'%(obj.name_append)
    density_swe_fig = 'density_swe%s.png'%(obj.name_append)
    
    if obj.tex_file == 'tuol_report_flt.tex':
        changes_flt_fig = 'swe_change_flt%s.png'%(obj.name_append)
        pre_swe = obj.pre_swe
        pre_pm = obj.pre_pm
        diff_pm = variables['TOTAL_PM'] - pre_pm
        
    if obj.units == 'SI':
        unitlbl = '$km^3$'
    else:
        unitlbl = vollbl
        
    # Upper case variables are used in the LaTex file, 
    # lower case versions are assigned here
    variables['REPORT_TITLE'] = report_title
    variables['REPORT_TIME'] = report_time
    variables['WATERYEAR'] = str(obj.wy)
    variables['UNITS'] = unitlbl
    variables['VOLLBL'] = vollbl
    variables['DEPLBL'] = deplbl
    variables['START_DATE'] = start_date
    variables['END_DATE'] = end_date
    variables['FORE_DATE'] = fore_date
    variables['SWE_IN'] = swe_in
    variables['SWI_IN'] = swi_in
    variables['FIG_PATH'] = fig_path
    variables['SWI_FIG'] = swi_fig
    variables['RESULTS_FIG'] = results_fig
    variables['CHANGES_FIG'] = changes_fig
    variables['ELEV_FIG'] = elev_fig
    variables['TOTALS_FIG'] = totals_fig
    variables['TOTALSMY_FIG'] = totalsmy_fig
    variables['HYP_FIG'] = hyp_fig         
    variables['MEAN_FIG'] = mean_fig
    variables['DETAIL_FIG'] = detail_fig 
    variables['DENSITY_FIG'] = density_fig    
    variables['DENSITY_SUB_FIG'] = density_sub_fig 
    variables['DENSITY_SWE_FIG'] = density_swe_fig                                  
    variables['VALID_FIG'] = valid_fig
        
    if obj.basin == 'TUOL' and obj.tex_file == 'tuol_report_flt.tex':
        variables['CHANGES_FLT_FIG'] = changes_flt_fig
        variables['PRE_SWE'] = pre_swe
        variables['PRE_PM'] = pre_pm
        variables['DIFF_PM'] = diff_pm
        variables['FLT_SWEDEL'] = variables['TOTAL_SWE'] - pre_swe
    
    # Convert floats to strings    
    for name in variables:
        if isinstance(variables[name], float):
            if obj.dplcs == 0:
                tmp = str(int(variables[name]))
            else:
                tmp = str(round(variables[name],obj.dplcs))
            variables[name] = tmp
  
    # Load in summary.txt files, and replace variables with values
    if obj.basin == 'BRB':
        summary = '%sbrb_summary.txt'%(obj.env_path)
        results_summary = '%sbrb_results_summary.txt'%(obj.env_path)
        
    if obj.basin == 'TUOL':
        if obj.tex_file == 'tuol_report_flt.tex':
            summary = '%stuol_summary_flt.txt'%(obj.env_path)
        else:
            summary = '%stuol_summary.txt'%(obj.env_path)
        results_summary = '%stuol_results_summary.txt'%(obj.env_path)

    if obj.basin == 'SJ':
        
        if obj.tex_file == 'sj_report_hrrr.tex':
            summary = '%ssj_summary_hrrr.txt'%(obj.env_path)
        else:
            summary = '%ssj_summary.txt'%(obj.env_path)
        
        results_summary = '%ssj_results_summary.txt'%(obj.env_path)
        
    if obj.basin == 'LAKES':
        summary = '%slakes_summary.txt'%(obj.env_path)
        results_summary = '%slakes_results_summary.txt'%(obj.env_path)   
    if obj.basin == 'RCEW':
        summary = '%srcew_summary.txt'%(obj.env_path)
        results_summary = '%srcew_results_summary.txt'%(obj.env_path)               
    
    # Read in both section summaries and then replace variables
    fid = open(summary,'r')
    fid1 = open(results_summary,'r')
    summary = fid.read()
    results_summary = fid1.read()
    fid.close()
    fid1.close()  
    
    for name in variables:
        summary = summary.replace(name,variables[name])         
        results_summary = results_summary.replace(name,variables[name]) 
            
    # Add the section text variables to what we'll pass to the document
    variables['SUMMARY'] = summary 
    variables['RESULTS_SUMMARY'] = results_summary 
           
    # Make the report
    env = make_env(loader = FileSystemLoader(obj.templ_path))
    tpl = env.get_template(obj.tex_file)        
    pdf = build_pdf(tpl.render(variables))
    # To see what the latex version >>> print(tpl.render(variables))
    
    # Save in reports and with figs
    print('Saving report to %s%s'%(obj.rep_path,obj.report_name))
    pdf.save_to('%s%s'%(obj.rep_path,obj.report_name)) 
    pdf.save_to('%s%s'%(obj.figs_path,obj.report_name)) 




from setuptools import setup, find_packages
from codecs import open
from os import path

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    # TODO: put package requirements here
]

setup_requirements = [
    # TODO(micahsandusky5): put setup requirements (distutils extensions, etc.) here
]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='snowav',
    version='0.1.0',
    description="Snow and Water Model Analysis and Visualization ",
    long_description=readme + '\n\n' + history,
    author="Mark Robertson",
    author_email='mark.robertson@ars.usda.gov',
    url='https://github.com/roberton-mark/SNOWAV',
    packages=['snowav'
			  ],


    include_package_data=True,
    package_data={'snowav':['./config/CoreConfig.ini']},
    scripts=['./scripts/snow.py'],
    install_requires=requirements,
    license="GPL-3.0",
    zip_safe=False,
    keywords='snowav',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    setup_requires=setup_requirements,

    entry_points={
       'console_scripts': ['snowav = scripts.snow:run',],
    }

)




import numpy as np
from matplotlib import pyplot as plt
import matplotlib.colors as mcolors
from mpl_toolkits.axes_grid1 import make_axes_locatable
import seaborn as sns
import copy
import cmocean
import matplotlib.patches as mpatches
from datetime import datetime
import pandas as pd
from matplotlib.dates import DateFormatter
  
    
    
def basin_total(snow):       
         
    sns.set_style('darkgrid')
    sns.set_context("notebook")
    
    plt.close(8)
    fig,(ax,ax1) = plt.subplots(num=8, figsize=snow.figsize, 
                                dpi=snow.dpi, nrows = 1, ncols = 2)
    
    snow.barcolors.insert(0,'black')
    
    if snow.basin == 'LAKES' or snow.basin == 'RCEW':
        plotorder = [snow.plotorder[0]]
    else:
        plotorder = snow.plotorder
    
    for iters,name in enumerate(plotorder):
        # name = snow.plotorder[0]
        # iters = 0
        snow.state_summary[name].plot(ax=ax, color = snow.barcolors[iters])             
        ax1.plot(snow.accum_summary[name], 
                 color = snow.barcolors[iters], label='_nolegend_')
    
    ax1.yaxis.set_label_position("right")
    ax1.set_xlim((datetime(snow.wy -1 , 10, 1),snow.dateTo))
    ax.set_xlim((datetime(snow.wy - 1, 10, 1),snow.dateTo))
    ax1.tick_params(axis='y')
    ax1.yaxis.tick_right()
    ax.legend(loc='upper left')
    
    # Put on the same yaxis
    swey = ax.get_ylim()
    swiy = ax1.get_ylim()
    
    if swey[1] < swiy[1]:
        ax1.set_ylim((-0.1,swiy[1]))
        ax.set_ylim((-0.1,swiy[1]))
                    
    if swey[1] >= swiy[1]:
        ax1.set_ylim((-0.1,swey[1]))
        ax.set_ylim((-0.1,swey[1]))
    
    for tick,tick1 in zip(ax.get_xticklabels(),ax1.get_xticklabels()):
        tick.set_rotation(30) 
        tick1.set_rotation(30) 
         
    ax1.set_ylabel(r'[%s]'%(snow.vollbl))
    ax.axes.set_title('Basin SWE')
    ax1.axes.set_title('Accumulated Basin SWI')
    ax.set_ylabel(r'[%s]'%(snow.vollbl)) 
    
    # plt.tight_layout()   
    del snow.barcolors[0] 
    
    print('saving figure to %sbasin_total%s.png'%(snow.figs_path,
                                                  snow.name_append))   
    plt.savefig('%sbasin_total%s.png'%(snow.figs_path,snow.name_append))       
    
    ########################################
    #         Second figure
    ########################################
    
    # This needs to be improved...
    
    if snow.basin == 'BRB':
        accum_summary = snow.accum_summary
        main = 'Boise River Basin'
        multiswe = pd.DataFrame.from_csv(snow.summary_swe) 
        multiswi = pd.DataFrame.from_csv(snow.summary_swi)
        
        multiswe.wy17.iloc[304:] = 0
        multiswi.wy17 = np.cumsum(multiswi.wy17)
        multiswi.wy17.iloc[304:] = multiswi.wy17[303]
        
        state_summary = snow.state_summary.asfreq('D')
        
        # Put in this year
        multiswe.wy18.iloc[:len(state_summary[main])] = (
                                            state_summary[main].values) 
        multiswi.wy18.iloc[:len(accum_summary[main])] = (
                                            accum_summary[main].values)          
        
        if snow.units == 'SI':
            multiswe.wy17 = np.multiply(multiswe.wy17,0.00123348)
            multiswi.wy17 = np.multiply(multiswi.wy17,0.00123348)
            multiswe.wy16 = np.multiply(multiswe.wy16,0.00123348)
            multiswi.wy16 = np.multiply(multiswi.wy16,0.00123348)
            multiswe.wy15 = np.multiply(multiswe.wy15,0.00123348)
            multiswi.wy15 = np.multiply(multiswi.wy15,0.00123348)                
        
        plt.close(8)
        fig,(ax,ax1)    = plt.subplots(num=8, figsize=snow.figsize,
                                       dpi=snow.dpi, nrows = 1, ncols = 2)
    
        ax.plot(multiswe['wy15'], color = 'g',label = 'wy2015')
        ax.plot(multiswe['wy16'], color = 'r',label = 'wy2016')
        ax.plot(multiswe['wy17'], color = 'k',label = 'wy2017')
        ax.plot(multiswe['wy18'], color = 'b', label = 'wy2018')
    
        ax1.plot(multiswi['wy15'], color = 'g',label = 'wy2015')
        ax1.plot(multiswi['wy16'], color = 'r',label = 'wy2016')
        ax1.plot(multiswi['wy17'], color = 'k',label = 'wy2017')
        ax1.plot(multiswi['wy18'], color = 'b', label = 'wy2018')
        
        formatter = DateFormatter('%b')
        ax.xaxis.set_major_formatter(formatter)
        ax1.xaxis.set_major_formatter(formatter)
    
        ax1.yaxis.set_label_position("right")
        ax1.set_xlim((datetime(2017, 10, 1),datetime(2018, 8, 1)))
        ax.set_xlim((datetime(2017, 10, 1),datetime(2018, 8, 1)))
        ax1.tick_params(axis='y')
        ax1.yaxis.tick_right()
        ax.legend(loc='upper left')
        
        for tick,tick1 in zip(ax.get_xticklabels(),ax1.get_xticklabels()):
            tick.set_rotation(30) 
            tick1.set_rotation(30) 
             
        if snow.units == 'KAF':
            ax.set_ylabel('[KAF]')  
            ax1.set_ylabel('[KAF]')
            ax.axes.set_title('Water Year SWE')
            ax1.axes.set_title('Accumulated Basin SWI')
        
        if snow.units == 'SI':
            ax.set_ylabel(r'[$km^3$]') 
            ax1.set_ylabel(r'[$km^3$]')            
            ax.axes.set_title('Basin SWE [$km^3$]')
            ax1.axes.set_title('Accumulated Basin SWI [$km^3$]')
        
        ax.set_ylim((0,ax1.get_ylim()[1]))
        plt.tight_layout()      
        
        print('saving figure to %sbasin_total_multiyr%s.png'%(snow.figs_path,snow.name_append))   
        plt.savefig('%sbasin_total_multiyr%s.png'%(snow.figs_path,snow.name_append))   
        
          



import numpy as np
from matplotlib import pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable
import seaborn as sns
from smrf import ipw
from shutil import copyfile
import os
import copy
import pandas as pd
import ConfigParser as cfp
import snowav.methods.wyhr_to_datetime as wy


class SNOWAV(object):

    def __init__(self,config_file):
        '''
        Notes: 

        '''
        
        try:
            if not os.path.isfile(config_file):
                print('Config file does not exist!')
                self.error = True
                return
                
            print('Reading the config file...')             
            cfg = cfp.ConfigParser()
            cfg.read(config_file)
    
            ####################################################
            #             Basin section                        #
            ####################################################
            self.basin = cfg.get('Basin','basin')
            self.save_path = cfg.get('Basin','save_path')
            self.name_append = cfg.get('Basin','name_append')
            self.wy = int(cfg.get('Basin','wy'))
            self.units = cfg.get('Basin','units') 
            
            ####################################################
            #           Outputs section                        #
            ####################################################        
            # Default for snow.0000 file is 2 (SWE)
            if (cfg.has_option('Outputs','snowband')): 
                self.snowband = int(cfg.get('Outputs','snowband'))     
            else:
                self.snowband = 2
            
            # Default for em.0000 is 8 (SWI)
            if (cfg.has_option('Outputs','emband')): 
                self.snowband = int(cfg.get('Outputs','emband'))     
            else:
                self.emband = 8           

            # Default rounding decimals for volume and depth 
            if (cfg.has_option('Outputs','decimals')): 
                self.dplcs = int(cfg.get('Outputs','decimals'))   
            else:
                self.dplcs = 1   
             
            # If psnowFile and csnowFile are specified, use those, 
            # otherwise they will get defined as the first and last
            # snow.XXXX file once [Runs] has been compiled
            if (cfg.has_option('Outputs','psnowFile') and 
                cfg.has_option('Outputs','csnowFile')):   
                self.psnowFile = cfg.get('Outputs','psnowFile')
                self.csnowFile = cfg.get('Outputs','csnowFile') 
                self.cemFile = self.csnowFile.replace('snow.','em.')         
            
            ####################################################
            #           Runs                                   #
            ####################################################       
            # Collect all the run directories
            self.run_dirs = list(cfg.items('Runs'))             
            self.snow_files = []
            self.em_files = []
            
            for rdir in self.run_dirs:

                run_files = [rdir[1] + s for s in sorted(os.listdir(rdir[1]))]
                
                self.snow_files = (self.snow_files 
                                   + [value for value in run_files 
                                   if ( ('snow.' in value) and not ('.nc' in value))])
                self.em_files = (self.em_files 
                                 + [value for value in run_files 
                                 if ( ('em.' in value) and not ('.nc' in value))])
            
            while '*snow.nc' in self.snow_files:
                self.snow_files.remove('*snow.nc')
            
            while '*em.nc' in self.em_files:
                self.em_files.remove('*em.nc')                
       
            # If no psnowFile and csnowFile specified, use first and last
            if not cfg.has_option('Outputs','csnowFile'):
                print('Using first and last outputs as start and end times.')
                self.psnowFile = self.snow_files[0] 
                self.csnowFile = self.snow_files[len(self.snow_files)-1] 
                self.cemFile = self.em_files[len(self.em_files)-1]
                
            # Check to see if they exist
            if not (os.path.isfile(self.psnowFile)):   
                print('psnowFile %s does not exist!'%(self.psnowFile))
                self.error = True
                return            
            
            if not (os.path.isfile(self.csnowFile)):   
                print('csnowFile does not exist!')
                self.error = True
                return
            
            ####################################################
            #           Validate                               #
            #################################################### 
            if (cfg.has_option('Validate','stations')
                and (cfg.has_option('Validate','labels'))
                and (cfg.has_option('Validate','client'))
                ):
                self.val_stns = cfg.get('Validate','stations').split(',')
                self.val_lbls = cfg.get('Validate','labels').split(',')
                self.val_client = cfg.get('Validate','client')   
                self.valid_flag = True
            else:
                self.valid_flag = False
                print('No validation stations listed, will not generate figure')
            
            # This is being used to combine 2017 HRRR data
            if cfg.has_option('Validate','offset'):
                self.offset = int(cfg.get('Validate','offset'))   
            else:
                self.offset = 0                    
            ####################################################
            #           Accumulated                            #
            ####################################################   
            # Right now we aren't using these because matplotlib and multiple
            # colormaps don't get along well with clims...
            # self.acc_clmin = cfg.get('Accumulated','clmin')
            # self.acc_clmax = cfg.get('Accumulated','clmax') 
            if (cfg.has_option('Accumulated','ymin') 
                and cfg.has_option('Accumulated','ymax')):
    
                self.acc_ylims = (int(cfg.get('Accumulated','ymin')),
                                  int(cfg.get('Accumulated','ymax')))  
            
            if cfg.has_option('Accumulated','save_fig'):
                self.acc_flag = cfg.get('Accumulated','save_fig')
            else:
                self.acc_flag = True

            if cfg.has_option('Accumulated','min_swi'):
                self.min_swi = cfg.get('Accumulated','min_swi')   
                       
    
            ####################################################
            #           Elevation                              #
            ####################################################             
            if (cfg.has_option('Elevation','ymin') 
                and cfg.has_option('Elevation','ymax')):
                self.elv_ylims = (int(cfg.get('Elevation','ymin')),
                                 int(cfg.get('Elevation','ymax')))  
            
            if cfg.has_option('Elevation','save_fig'):
                self.elv_flag = cfg.get('Elevation','save_fig')
            else:
                self.elv_flag = True                                   
            
            ####################################################
            #           Changes                                #
            ####################################################           
            self.ch_clmin = cfg.get('Changes','clmin')
            self.ch_clmax = cfg.get('Changes','clmax')
                 
            if (cfg.has_option('Changes','clminabs') 
                and cfg.has_option('Changes','clmaxabs')):   
                self.ch_clminabs = int(cfg.get('Changes','clminabs'))
                self.ch_clmaxabs = int(cfg.get('Changes','clmaxabs'))
            if (cfg.has_option('Changes','ymin') 
                and cfg.has_option('Changes','ymax')):
                self.ch_ylims = (int(cfg.get('Changes','ymin')),
                                 int(cfg.get('Changes','ymax')))            
            
            ####################################################
            #           Results                                #
            ####################################################           
            self.r_clmin = cfg.get('Results','clmin')
            self.r_clmax = cfg.get('Results','clmax')     

            ####################################################
            #           Basin Total                            #
            ####################################################          
            if (cfg.has_option('Basin Total','summary_swe') and 
                cfg.has_option('Basin Total','summary_swi')):
                self.summary_swe = cfg.get('Basin Total','summary_swe')
                self.summary_swi = cfg.get('Basin Total','summary_swi')  
            if not (os.path.isfile(self.summary_swe) and 
                    os.path.isfile(self.summary_swi) ):   
                print('Failed reading in Basin Total section!')
                self.error = True
                return 
            
            #
            if cfg.has_option('Basin Total','netcdf'):
                self.ncvars = cfg.get('Basin Total','netcdf').split(',')     
                self.nc_flag = True
            else:
                self.nc_flag = False    
            
            ####################################################
            #           DEM                                    #
            ####################################################         
            self.demPath = cfg.get('DEM','demPath')
            self.total = cfg.get('DEM','total')     
    
            ####################################################
            #          Plots                                   #
            ####################################################  
            self.figsize = (int(cfg.get('Plots','fig_length')),
                            int(cfg.get('Plots','fig_height')))
            self.dpi = int(cfg.get('Plots','dpi'))
            self.barcolors = ['xkcd:true green','palegreen',
                              'xkcd:dusty green','xkcd:vibrant green','red']     
    
            ####################################################
            #          Report                                  #
            ####################################################    
            self.report_flag = cfg.get('Report','report')  
            self.env_path = cfg.get('Report','env_path')
            self.tex_file = cfg.get('Report','tex_file')
            self.rep_path = cfg.get('Report','rep_path')
            self.templ_path = cfg.get('Report','templ_path')
            
            if cfg.has_option('Report','orig_date'):
                self.orig_date = cfg.get('Report','orig_date')
            if cfg.has_option('Report','report_name_append'):
                self.rep_append = cfg.get('Report','report_name_append')                
            
            # These will later get appended with self.dateTo 
            self.report_name = cfg.get('Report','report_name')
            self.rep_title = cfg.get('Report','report_title') 

            if not (os.path.isfile(self.templ_path + self.tex_file)):
                print('Error reading in Reports section!')
                self.error = True
                return                  
              
            # Strings for the report
            if self.units == 'KAF':
                self.reportunits = 'KAF' 
            if self.units == 'SI':
                self.reportunits = 'km^3'
                           
            ####################################################  
            # History forecast
            ####################################################  
            if cfg.has_option('Hx Forecast','adj_hours'):
                self.adj_hours = int(cfg.get('Hx Forecast','adj_hours'))
            
            ####################################################   
            
            # Get the DEM 
            # There are different formats, this will get fixed once we
            # start using netcdf
            try: 
                self.dem = np.genfromtxt(self.demPath)
            except:
                self.dem = np.genfromtxt(self.demPath,skip_header = 6)
            
            self.nrows = len(self.dem[:,0])
            self.ncols = len(self.dem[0,:]) 
            blank = np.zeros((self.nrows,self.ncols))
                           
            # Assign some basin-specific things
            if self.basin == 'BRB':
                self.pixel = 100
                sr = 0
                if self.units == 'KAF':
                    emin = 2500      
                    emax = 10500     
                    self.step = 1000
                if self.units == 'SI':
                    emin = 800       
                    emax = 3200       
                    self.step = 250                 
            if self.basin == 'TUOL':
                self.pixel = 50   
                sr = 6
                if self.units == 'KAF':
                    emin = 3000      # [ft]
                    emax = 12000    
                    self.step   = 1000
                if self.units == 'SI':
                    emin = 800       # [m]
                    emax = 3600     
                    self.step   = 500                  
            if self.basin == 'SJ':
                self.pixel = 50
                sr = 6
                if self.units == 'KAF':
                    emin = 1000      # [ft]
                    emax  = 13000   
                    self.step = 1000
                if self.units == 'SI':
                    emin = 300       # [m]
                    emax = 4000       
                    self.step = 500                 
            if self.basin == 'LAKES':
                self.pixel = 50  
                sr = 0
                self.imgx = (1200,1375)
                self.imgy = (425,225) 
                if self.units == 'KAF':
                    emin = 8000      # [ft]
                    emax = 12500      
                    self.step = 500
                if self.units == 'SI':
                    emin = 2400       # [m]
                    emax = 3800      
                    self.step = 200                    
            if self.basin == 'RCEW':
                self.pixel = 50
                sr = 0
                if self.units == 'KAF':
                    emin = 2500      # [ft]
                    emax = 7500       
                    self.step = 500
                if self.units == 'SI':
                    emin = 800       # [m]
                    emax = 2500     
                    self.step = 500  
     
            # This is for creating the elevation bins
            self.edges = np.arange(emin,emax+self.step,self.step)
            # Right now this is a placeholder, could edit by basin...
            self.xlims = (0,len(self.edges))
            
            try:
                self.subbasin1 = cfg.get('DEM','subbasin1')
                self.subbasin2 = cfg.get('DEM','subbasin2')
                self.subbasin3 = cfg.get('DEM','subbasin3')
                self.total_lbl = cfg.get('DEM','total_lbl')
                self.sub1_lbl = cfg.get('DEM','sub1_lbl')
                self.sub2_lbl = cfg.get('DEM','sub2_lbl')
                self.sub3_lbl = cfg.get('DEM','sub3_lbl') 
                
                self.plotorder = [self.total_lbl, self.sub1_lbl, 
                                  self.sub2_lbl, self.sub3_lbl]  
                self.suborder = [self.sub1_lbl,self.sub2_lbl,self.sub3_lbl]  
                maskpaths = [self.total, self.subbasin1, 
                             self.subbasin2,self.subbasin3 ]
    
                # Add if necessary - need to generalize all this and change 
                # in the config file!
                if cfg.has_option('DEM','sub4_lbl'): 
                    self.sub4_lbl = cfg.get('DEM','sub4_lbl') 
                    self.subbasin4 = cfg.get('DEM','subbasin4')
                    self.plotorder = self.plotorder + [self.sub4_lbl]   
                    self.suborder = self.suborder + [self.sub4_lbl]                        
                    maskpaths = maskpaths + [self.subbasin4]
                
                # Compile the masks 
                # HACK FOR SJ SUB4!
                self.masks = dict()
                for lbl,mask in zip(self.plotorder,maskpaths):
                    if (self.basin == 'SJ' and lbl == self.sub4_lbl):
                        self.masks[lbl] = {'border': blank, 
                                           'mask': np.genfromtxt(mask,skip_header=0),
                                           'label': lbl}   
                    # hack
                    else:
                        self.masks[lbl] = {'border': blank, 
                                           'mask': np.genfromtxt(mask,skip_header=sr),
                                           'label': lbl}                              
            except:
                print('Error creating mask dicts!')
                self.error = True
                return
                  
            # Do unit-specific things
            if self.units == 'KAF':
                self.conversion_factor = ((self.pixel**2)
                                         * 0.000000810713194*0.001) # [KAF]   
                self.depth_factor = 0.03937 # [inches]
                self.dem = self.dem * 3.28 # [ft]
                self.ixd = np.digitize(self.dem,self.edges)     
                self.depthlbl = 'in'
                self.vollbl = 'KAF' 
                self.elevlbl = 'ft'   
            
            if self.units == 'SI':
                # [km^3]
                self.conversion_factor = ((self.pixel**2)
                                          * 0.000000810713194*1233.48/1e9) 
                self.depth_factor = 1 # [m]                                                
                self.ixd = np.digitize(self.dem,self.edges)     
                self.depthlbl = 'mm'
                self.vollbl = '$km^3$' 
                self.elevlbl = 'm'     
                                 
            # Copy the config file where figs will be saved
            extf = os.path.splitext(os.path.split(config_file)[1])
            path_shr = os.path.split(self.psnowFile)
            path_ehr = os.path.split(self.csnowFile)
            ext_shr = os.path.splitext(path_shr[1])
            ext_ehr = os.path.splitext(path_ehr[1])
            self.figs_path = (self.save_path 
                             + '%s_%s/'%(ext_shr[1][1:5],ext_ehr[1][1:5]))
            
            # Check run dates, and make a new folder if necessary
            if not os.path.exists(self.figs_path):
                os.makedirs(self.figs_path)
    
            # Only need to store this name if we decide to 
            # write more to the copied config file...
            self.config_copy = (self.figs_path 
                                + extf[0]
                                + self.name_append
                                + '_%s_%s'%(ext_shr[1][1:5],ext_ehr[1][1:5]) 
                                + extf[1])
            
            # If it doesn't already exist, make it
            if not os.path.isfile(self.config_copy): 
                copyfile(config_file,self.config_copy)
        
        except:
            print('Error reading config file.')

    def process(self,*args):
        '''
        This function calculates everything we will need for the plots.
        
        Does not currently save to csv...

        '''
  
        print('Processing iSnobal outputs...')
        
        # If we add pre and current snow files for flights, force those
        # and add a few new things to calculate
        if len(args) != 0:
            self.psnowFile = args[0]
            self.csnowFile = args[1]
            self.cemFile = args[1].replace('snow','em')
            self.snow_files = [self.psnowFile, self.csnowFile]
            self.em_files = [self.psnowFile.replace('snow','em'), 
                             self.csnowFile.replace('snow','em')]
       
        cclimit = -5*1000*1000  # based on an average of 60 W/m2 from TL paper
        # ccM = cc./1000./1000; % cold content in MJ

        accum = np.zeros((self.nrows,self.ncols))
        evap = copy.deepcopy(accum)
        accum_sub = copy.deepcopy(accum)
        snowmelt = copy.deepcopy(accum)
        rain_bg = copy.deepcopy(accum)
        precip = copy.deepcopy(accum)
        precip_sub = copy.deepcopy(accum)
        snowmelt_sub =copy.deepcopy(accum)
        state = copy.deepcopy(accum)
        depth = copy.deepcopy(accum)
        pstate = copy.deepcopy(accum)
        state_byday = np.zeros((self.nrows,self.ncols,len(self.snow_files)))
        
        accum_byelev = pd.DataFrame(index = self.edges, 
                                    columns = self.masks.keys())
        evap_byelev = copy.deepcopy(accum_byelev)
        rain_bg_byelev = copy.deepcopy(accum_byelev)
        precip_byelev = copy.deepcopy(accum_byelev)
        precip_byelev_sub = copy.deepcopy(accum_byelev)
        accum_byelev_sub = copy.deepcopy(accum_byelev)  
        state_byelev = copy.deepcopy(accum_byelev)
        density_m_byelev = copy.deepcopy(accum_byelev)
        depth_byelev = copy.deepcopy(accum_byelev)
        state_mswe_byelev = copy.deepcopy(accum_byelev)
        depth_mdep_byelev = copy.deepcopy(accum_byelev)
        delta_state_byelev = copy.deepcopy(accum_byelev)
        melt = copy.deepcopy(accum_byelev)
        nonmelt = copy.deepcopy(accum_byelev)
        snowmelt_byelev = copy.deepcopy(accum_byelev)
        snowmelt_byelev_sub = copy.deepcopy(accum_byelev)
        
        state_summary = pd.DataFrame(columns = self.masks.keys())
        accum_summary = pd.DataFrame(columns = self.masks.keys())
        precip_summary = pd.DataFrame(columns = self.masks.keys())
        evap_summary = pd.DataFrame(columns = self.masks.keys())
               
        # Loop through output files
        # Currently we need to load in each em.XXXX file to 'accumulate',
        # but only the first and last snow.XXXX file for changes
        accum_sub_flag = False
        adj = 0 # this is a hack for Hx-repeats-itself forecasting
        t = 0
        for iters,(em_name,snow_name) in enumerate(zip(self.em_files,
                                                       self.snow_files)):
            # iters = 0
            # iters = iters + 1
            # em_name = self.em_files[iters]
            # snow_name = self.snow_files[iters]        

            date = wy.wyhr_to_datetime(self.wy,
                                       int(snow_name.split('.')[-1])
                                       + adj)  
            
            # It's nice to see where we are...
            print(snow_name, int(snow_name.split('.')[-1]) - t )
            print(date)
            
            # Hack for Hx-repeats-itself forecasting
            if snow_name == self.psnowFile:
                if hasattr(self,"adj_hours"):
                    print('Hacking adj_hours...')
                    adj = self.adj_hours
                   
            em_file = ipw.IPW(em_name)
            band = em_file.bands[self.emband].data
            accum = accum + band
            snowmelt = snowmelt + em_file.bands[7].data
            evap = evap + em_file.bands[6].data
 
            # load and calculate sub-basin total
            snow_file = ipw.IPW(snow_name)
            tmpstate = snow_file.bands[self.snowband].data  
            state_byday[:,:,iters] = tmpstate
            
            # Get rain from input data
            sf = em_name.replace('runs','data')
            sf = sf.replace('run','data')
            sf = sf.replace('output','ppt_4b')
            ppt_path = sf.split('em')[0]
            
            out_hr = snow_name.split('.')[-1]
            hrs = range(int(out_hr) - 23,int(out_hr) + 1)
            
            pFlag = False
            ppt_files = []
            for hr in hrs:
                if hr < 100:
                    shr = '00' + str(hr)
                    
                if hr >=100 and hr < 1000:
                    shr = '0' + str(hr)  
                    
                if hr >= 1000:
                    shr = str(hr)     
                
                if os.path.isfile(ppt_path +'ppt.4b_'+ shr):
                    pFlag = True
                    ppt_files = ppt_files + [ppt_path +'ppt.4b_'+ shr]
            
            rain_hrly = np.zeros((self.nrows,self.ncols))
            precip_hrly = np.zeros((self.nrows,self.ncols))
            
            # Load 'em in
            if pFlag and accum_sub_flag == True:
                print('adding precip for %s'%(snow_name))
                for pfile in ppt_files:
                    ppt = ipw.IPW(pfile)
                    pre = ppt.bands[0].data
                    percent_snow = ppt.bands[1].data
                    rain_hrly = rain_hrly + np.multiply(pre,(1-percent_snow))
                    precip_hrly = precip_hrly + pre
            else:
                print(pFlag,accum_sub_flag)
            
            rain_bg = rain_bg + rain_hrly 
            precip = precip + precip_hrly    
                
            # Currently this is grabbing the second to last
            # Being used for flight difference
            # Definitely a better way to do this...
            if iters == (len(self.snow_files) - 1):
                # Mean pixel depth [mm] on psnowFile
                self.pre_pm = (
                               np.nansum(tmpstate 
                               * self.masks[self.total_lbl]['mask'])
                               / self.masks[self.total_lbl]['mask'].sum() 
                               )  
                self.pre_swe = ( 
                                np.nansum(tmpstate 
                                * self.masks[self.total_lbl]['mask'])
                                * self.conversion_factor )      
            
            # Store daily sub-basin totals
            for name in self.masks:
                accum_summary.loc[date, name] = (np.nansum(
                                                np.multiply(accum,
                                                self.masks[name]['mask'])) )
                state_summary.loc[date, name] = (np.nansum(
                                                np.multiply(tmpstate,
                                                self.masks[name]['mask'])) )
                precip_summary.loc[date, name] = (np.nansum(
                                                np.multiply(precip,
                                                self.masks[name]['mask'])) )
                evap_summary.loc[date, name] = (np.nansum(
                                                np.multiply(evap,
                                                self.masks[name]['mask'])) )                

            # This only add between psnowFile and csnowFile
            if accum_sub_flag:
                accum_sub = accum_sub + copy.deepcopy(band)
                snowmelt_sub = ( snowmelt_sub 
                                + copy.deepcopy(em_file.bands[7].data) ) 
                precip_sub = precip_sub + precip_hrly            
           
            # When it is the first snow file, copy
            if snow_name == self.psnowFile:
                print('psnowfile is %s'%(snow_name))
                accum_sub_flag = True
                pstate = copy.deepcopy(tmpstate)           
            
            # When it hits the current snow file, copy
            if snow_name == self.csnowFile:
                print('csnowfile is %s'%(snow_name))
                
                # Turn off, but last one will still have been added
                accum_sub_flag  = False
             
                state = copy.deepcopy(tmpstate)
                depth = snow_file.bands[0].data
                density = snow_file.bands[1].data
                self.density = copy.deepcopy(density)
                self.cold = em_file.bands[9].data   
                
                # No need to compile more files after csnowFile
                break     

            t = int(snow_name.split('.')[-1])

        self.dateFrom = wy.wyhr_to_datetime(self.wy,
                                            int(self.psnowFile.split('.')[-1]))
        self.dateTo = wy.wyhr_to_datetime(self.wy,
                                          int(self.csnowFile.split('.')[-1]) 
                                          + adj)            
 
        # Append date to report name
        parts = self.report_name.split('.')
        self.report_name = ( parts[0] 
                           + self.dateTo.date().strftime("%Y%m%d") 
                           + '.' + parts[1] )
        
        # Difference in state (SWE)
        delta_state         = state - pstate
        
        # Mask by subbasin and elevation band
        for name in self.masks:
            mask = copy.deepcopy(self.masks[name]['mask'])
            
            accum_mask = np.multiply(accum,mask)
            evap_mask = np.multiply(evap,mask)
            rain_bg_mask = np.multiply(rain_bg,mask)
            precip_mask = np.multiply(precip,mask)
            precip_sub_mask = np.multiply(precip_sub,mask)
            accum_mask_sub = np.multiply(accum_sub,mask)
            snowmelt_mask = np.multiply(snowmelt,mask)
            snowmelt_mask_sub = np.multiply(snowmelt_sub,mask)
            state_mask = np.multiply(state,mask)
            delta_state_byelev_mask = np.multiply(delta_state,mask)
            state_byelev_mask = np.multiply(state,mask)
            state_mswe_byelev_mask = np.multiply(state,mask)
            density_m_byelev_mask = np.multiply(density,mask)
            
            ix = density_m_byelev_mask == 0
            density_m_byelev_mask[ix] = np.nan
            
            depth_mdep_byelev_mask = np.multiply(depth,mask)
            elevbin = np.multiply(self.ixd,mask)
            
            # Do it by elevation band
            for n in np.arange(0,len(self.edges)):
                ind = elevbin == n
                state_bin = state_mask[ind]
                b = self.edges[n]
                
                # Cold content
                ccb = self.cold[ind]
                cind = ccb > cclimit
                
                accum_byelev.loc[b,name] = np.nansum(accum_mask[ind])
                evap_byelev.loc[b,name] = np.nansum(evap_mask[ind])
                rain_bg_byelev.loc[b,name] = np.nansum(rain_bg_mask[ind])
                precip_byelev.loc[b,name] = np.nansum(precip_mask[ind])
                precip_byelev_sub.loc[b,name] = np.nansum(precip_sub_mask[ind])
                accum_byelev_sub.loc[b,name] = np.nansum(accum_mask_sub[ind])
                snowmelt_byelev.loc[b,name] = np.nansum(snowmelt_mask[ind])
                state_byelev.loc[b,name] = np.nansum(state_byelev_mask[ind])
                depth_byelev.loc[b,name] = np.nansum(state_byelev_mask[ind])  
                melt.loc[b,name] = np.nansum(state_bin[cind])
                nonmelt.loc[b,name] = np.nansum(state_bin[~cind])   
                snowmelt_byelev_sub.loc[b,name] = np.nansum(
                                                snowmelt_mask_sub[ind])
                
                
                # Calculate mean if there is snow 
                if state_mswe_byelev_mask[ind].size:
                    state_mswe_byelev.loc[b,name] = np.nanmean(
                                                state_mswe_byelev_mask[ind])
                    depth_mdep_byelev.loc[b,name] = np.nanmean(
                                                depth_mdep_byelev_mask[ind])
                    delta_state_byelev.loc[b,name] = np.nansum(
                                                delta_state_byelev_mask[ind])
                else:
                    state_mswe_byelev.loc[b,name] = np.nan
                    depth_mdep_byelev.loc[b,name] = np.nan
                    delta_state_byelev.loc[b,name] = np.nan
                
            self.masks[name]['SWE'] = ( (melt[name].sum() + nonmelt[name].sum())
                                       * self.conversion_factor )      
        
        # Convert to desired units
        
        # Sum over all time steps, spatial
        self.precip = np.multiply(precip,self.depth_factor)                
        self.accum = np.multiply(accum,self.depth_factor)     
        self.accum_sub = np.multiply(accum_sub,self.depth_factor)   
        
        # At last time step
        self.state_byelev = np.multiply(state_byelev,self.conversion_factor)
        self.depth_byelev = np.multiply(depth_byelev,self.conversion_factor)         
        self.state = np.multiply(state,self.depth_factor) 
        self.depth = np.multiply(np.multiply(depth,1000),self.depth_factor) 
        self.delta_state = np.multiply(delta_state,self.depth_factor)  
        
        # Sum, by elevation
        self.accum_byelev = np.multiply(accum_byelev,self.conversion_factor) 
        self.evap_byelev = np.multiply(evap_byelev,self.conversion_factor) 
        self.precip_byelev = np.multiply(precip_byelev,self.conversion_factor) 
        self.rain_bg_byelev = np.multiply(rain_bg_byelev,
                                          self.conversion_factor)
        self.snowmelt_byelev = np.multiply(snowmelt_byelev,
                                           self.conversion_factor)  
        
        # Sub-set of time defined by psnowfile and csnowfile, sum, by elevation
        self.snowmelt_byelev_sub = np.multiply(snowmelt_byelev_sub,
                                               self.conversion_factor)  
        self.accum_byelev_sub = np.multiply(accum_byelev_sub,
                                            self.conversion_factor)      
        self.precip_byelev_sub = np.multiply(precip_byelev_sub,
                                             self.conversion_factor)
        
        # Change, by elevation
        self.delta_state_byelev = np.multiply(delta_state_byelev,
                                              self.conversion_factor) 
        
        # At first time step, spatial
        self.pstate = np.multiply(pstate,self.conversion_factor)
        
        # Daily 
        self.state_summary = np.multiply(state_summary,self.conversion_factor)       
        self.accum_summary = np.multiply(accum_summary,self.conversion_factor)         
        self.precip_summary = np.multiply(precip_summary,self.conversion_factor) 
        self.evap_summary = np.multiply(evap_summary,self.conversion_factor)                                
        self.state_byday = np.multiply(state_byday,self.depth_factor)  
    
        # Mean      
        self.state_mswe_byelev = np.multiply(state_mswe_byelev,
                                             self.depth_factor)
        self.depth_mdep_byelev = np.multiply(np.multiply(depth_mdep_byelev,1000)
                                             ,self.depth_factor)         
        self.density_m_byelev = density_m_byelev
        
        self.melt = np.multiply(melt,self.conversion_factor)
        self.nonmelt = np.multiply(nonmelt,self.conversion_factor)               
        self.cold = np.multiply(self.cold,0.000001)                                    
        
        # Let's print some summary info...
        mask        = self.state_summary.index.to_series().diff() > pd.Timedelta('24:10:00')
        msum        = sum(mask)
        print('%s entries in dataframe index with gaps larger than 24h '%(msum))
        print('done.')

    def basin_detail(self):
        import math
        import copy
        import numpy as np
        
        demcopy = copy.copy(self.dem)
        pmask           = self.masks[self.total_lbl]['mask']
        ixo             = pmask == 0
        demcopy[ixo]      = np.nan

        emin        = int(math.ceil(np.nanmin(demcopy)/10.0))*10
        emax        = int(math.ceil(np.nanmax(demcopy)/10.0))*10
        step        = 50 # 50
        edges       = np.arange(emin,emax+step,step)
        ixd         = np.digitize(demcopy,edges) 
        hypsom      = pd.DataFrame(index = edges, columns = self.masks.keys())  

        for name in self.masks:
            elevbin     = np.multiply(ixd,self.masks[name]['mask'])

            # Do it by elevation band
            for n in np.arange(0,len(edges)):
                ind        = elevbin == n    
                hypsom.loc[edges[n],name] = (np.nansum(np.nansum(ind))/np.nansum(np.nansum(self.masks[name]['mask'])))*100
            
        
        for name in self.masks:
            smin = np.nanmin(self.dem[self.dem*self.masks[name]['mask'] > 0])
            smax = np.nanmax(self.dem[self.dem*self.masks[name]['mask'] > 0])
            print(name,smin,smax)
        
        # filter out high and low hyp
        map     = plt.cm.terrain
        map.set_bad('white')
        
        sns.set_style('darkgrid')
        sns.set_context("notebook")
       
        clrs = copy.copy(self.barcolors)
        clrs.insert(0,'k')
       
        plt.close(10) 
        fig,(ax,ax1)    = plt.subplots(num=10, figsize=self.figsize, dpi=self.dpi, nrows = 1, ncols = 2)
        h               = ax.imshow(demcopy, cmap=map)
        
        # Basin boundaries
        for name in self.masks:
            ax.contour(self.masks[name]['mask'],cmap = "Greys",linewidths = 1)   

        for iters,name in enumerate(self.plotorder):
            hyp = copy.deepcopy(hypsom[name].cumsum()-hypsom[name].iloc[0])
            imin = hyp < 0.1
            hyp[imin] = np.nan
            imax = hyp > 99.9
            hyp[imax] = np.nan
            
            # ax1.plot(hypsom[name].cumsum()[::-1]-hypsom[name].iloc[0],range(0,len(edges)),color = clrs[iters],label = name) 
            ax1.plot(hyp,range(0,len(edges)),color = clrs[iters],label = name) 
         
        ax1.invert_xaxis()
        ax1.legend()
        ax1.set_xlim((102,-2))
        ax1.set_ylim((-1,len(hypsom)))
        
        ax1.set_ylabel('elevation [m]')
        ax1.yaxis.tick_right()
        ax1.yaxis.set_label_position("right")
        
        labels = ax1.get_xticklabels()
        ax1.set_xticklabels(labels[::-1])
              
        xts         = ax1.get_yticks()
        edges_lbl   = []
        for i in xts[0:len(xts)-1]:
            edges_lbl.append(str(int(edges[int(i)])))        
        
        ax1.set_yticklabels(str(i) for i in edges_lbl)
        
        ax1.set_xlabel(r'Basin area above elevation [%]')
        ax.set_title(self.plotorder[0])    

        h.axes.get_xaxis().set_ticks([])
        h.axes.get_yaxis().set_ticks([])
        divider = make_axes_locatable(ax)
        cax     = divider.append_axes("left", size="5%", pad=0.2)
        cbar    = plt.colorbar(h, cax = cax)
        cax.yaxis.set_ticks_position('left')
        cax.yaxis.set_label_position('left')
        cax.set_ylabel('elevation [m]')
        
        plt.subplots_adjust(top=0.88)
        plt.tight_layout() 
        
        if self.basin == 'SJ':
            ax.text(0.3,0.88,'Main',horizontalalignment='center',transform=ax.transAxes,fontsize = 10)  
            ax.text(0.48,0.055,'Jose Creek',horizontalalignment='center',transform=ax.transAxes,fontsize = 10)
            ax.text(0.95,0.65,'South\nFork',horizontalalignment='center',transform=ax.transAxes,fontsize = 10)
            ax.text(0.05,0.55,'Willow\nCreek',horizontalalignment='center',transform=ax.transAxes,fontsize = 10)  
        if self.basin == 'BRB':
            ax.text(0.2,0.74,'Mores Creek',horizontalalignment='center',transform=ax.transAxes,fontsize = 10)  
            ax.text(0.48,0.77,'Twin\nSprings',horizontalalignment='center',transform=ax.transAxes,fontsize = 10)
            ax.text(0.88,0.6,'Featherville',horizontalalignment='center',transform=ax.transAxes,fontsize = 10) 
        if self.basin == 'Tuol':
            ax.text(0.75,0.68,'Tuolumne',horizontalalignment='center',transform=ax.transAxes,fontsize = 10)  
            ax.text(0.2,0.84,'Cherry',horizontalalignment='center',transform=ax.transAxes,fontsize = 10)
            ax.text(0.14,0.35,'Eleanor',horizontalalignment='center',transform=ax.transAxes,fontsize = 10)                                 
        
        print('saving figure to %shypsometry%s.png'%(self.figs_path,self.name_append))   
        plt.savefig('%shypsometry%s.png'%(self.figs_path,self.name_append))   

    
    def distribution_detail(self,*args): 
        '''
        Maybe get rid of this one, not using at this point...
        ''' 
                
        colors  = ['xkcd:rose red','xkcd:cool blue']
        
        if len(args) != 0:
            if args[0] == 'depth':
                name = self.plotorder[args[1]]
                print(name)
                
                value = self.state_mswe_byelev
                lim     = np.max(value[self.total_lbl]) 
                ylim    = np.max(lim) + np.max(lim)*0.15 

            elif args[0] == 'volume':
                value   =  self.melt
                value2  =  self.nonmelt 
                lim     = np.max(value[self.total_lbl]) + np.max(value[self.total_lbl])
                ylim    = np.max(lim) + np.max(lim)*0.1                        
        
        sns.set_style('darkgrid')
        
        plt.close(5)
        fig  = plt.figure(num=5, figsize=(self.figsize[0]*0.9,self.figsize[1]), dpi=self.dpi)
        ax = plt.gca()
     
        if 'value2' in locals():
            ax.bar(range(0,len(self.edges)),value[name], color = colors[0], bottom = self.nonmelt[name])
            ax.bar(range(0,len(self.edges)),value2[name], color = colors[1], label = 'unavail ') 
            
            # Get basin total storage in strings for label
            kaf    = str(np.int(sum(self.melt[name]) + sum(self.nonmelt[name]))) 
                
            if self.dplcs == 0:
                kaf    = str(np.int(sum(self.melt[name]) + sum(self.nonmelt[name]))) 
            else: 
                kaf    = str(np.round(sum(self.melt[name]) + sum(self.nonmelt[name]),self.dplcs))             
    
            ax.text(.25,0.95,'%s - %s %s'%(self.masks[name]['label'],kaf,self.vollbl),horizontalalignment='center',transform=ax.transAxes)
            ax.set_ylabel('%s'%(self.vollbl))
                  
                
            lbl = []
            for n in (0,1):
                if n == 0:
                    if self.dplcs == 0:
                        kafa = str(np.int(sum(self.melt[name]))) 
                    else: 
                        kafa = str(np.round(sum(self.melt[name]),self.dplcs)) 
                    
                    tmpa = ('avail = %s')%(kafa)
                                          
                    lbl.append(tmpa)
                    
                # kafna = str(np.int(sum(self.nonmelt[name]))) 
                if self.dplcs == 0:
                    kafna = str(np.int(sum(self.nonmelt[name]))) 
                else: 
                    kafna = str(np.round(sum(self.nonmelt[name]),self.dplcs))            
    
                tmpna = ('unavail = %s')%(kafna)                     
                lbl.append(tmpna) 
                
            ax.legend(lbl, loc = (0.025, 0.8),fontsize = 9)    
            fig.suptitle('SWE, %s'%self.dateTo.date().strftime("%Y-%-m-%-d"))                                                   
        
        else:
            ax.bar(range(0,len(self.edges)),value[name], color = 'xkcd:peacock blue')  
               
            ax.set_ylabel('Mean SWE [%s]'%(self.depthlbl)) 
            fig.suptitle('%s, Mean SWE, %s'%(name,self.dateTo.date().strftime("%Y-%-m-%-d")) ) 
      
        
        ax.set_xlim(self.xlims)                          
        xts         = ax.get_xticks()
        edges_lbl   = []
        for i in xts[0:len(xts)-1]:
            edges_lbl.append(str(int(self.edges[int(i)])))

        ax.set_xticklabels(str(i) for i in edges_lbl)
        ax.set_xlabel('elevation [%s]'%(self.elevlbl))
        
        ax.set_ylim((0,ylim)) 
        for tick in ax.get_xticklabels():
            tick.set_rotation(30)             
        
        fig.tight_layout()
        ax.set_xticks(xts)
        ax.set_xlim(self.xlims)
  
        fig.subplots_adjust(top=0.92,wspace = 0.1)  

        print('saving figure to %smean_detail%s.png'%(self.figs_path,self.name_append))
        plt.savefig('%smean_detail%s.png'%(self.figs_path,self.name_append))  
    
    def write_summary(self,df):

        # df          = 'accum_byelev'
        
        # dataframes  = ['accum_byelev','state_byelev','delta_state_byelev','melt','nonmelt','snowmelt_byelev','state_summary','accum_summary']
        dataframes  = ['accum_summary','state_summary']
        
        if df not in dataframes:
            print('df needs to be one of: %s'%(dataframes)) 
            # break
        
        print('Writing summary file to %s%s_summary.csv'%(self.figs_path,df))   
         
        if df == 'state_summary':
            self.state_summary.to_csv('%s%s_summary.csv'%(self.figs_path,df))
        if df == 'accum_summary':
            self.accum_summary.to_csv('%s%s_summary.csv'%(self.figs_path,df))            
        


from snowav.methods.MidpointNormalize import MidpointNormalize
import numpy as np
from matplotlib import pyplot as plt
import matplotlib.colors as mcolors
from mpl_toolkits.axes_grid1 import make_axes_locatable
import seaborn as sns
import copy
import cmocean
import matplotlib.patches as mpatches

def image_change(snow): 
    '''
    This plots snow.delta_state
    
    Should add in more functionality for plotting differences between various images/runs
    
    '''  
    
    # if len(args) != 0:
    #     snow.name_append = snow.name_append + args[0]
          
    # Make copy so that we can add nans for the plots, but not mess up the original
    delta_state = copy.deepcopy(snow.delta_state)
    qMin,qMax = np.percentile(delta_state,[1,99.5])
    
    ix = np.logical_and(delta_state < qMin, delta_state >= np.nanmin(np.nanmin(delta_state)))
    delta_state[ix] = qMin + qMin*0.2
    vMin,vMax = np.percentile(delta_state,[1,99])
    # clims = (qMin,qMax )
       
    # Override if absolute limits are provide in the config
    if hasattr(snow,'ch_clminabs') and hasattr(snow,'ch_clmaxabs'):
        clims       = (snow.ch_clminabs,snow.ch_clmaxabs)
 
    colorsbad = plt.cm.Accent_r(np.linspace(0., 1, 1))           
    colors1 = cmocean.cm.matter_r(np.linspace(0., 1, 127))
    colors2 = plt.cm.Blues(np.linspace(0, 1, 128))
    colors = np.vstack((colorsbad,colors1, colors2))
    mymap = mcolors.LinearSegmentedColormap.from_list('my_colormap', colors)

    ixf = delta_state == 0
    delta_state[ixf] = -100000 # set snow-free  
    pmask = snow.masks[snow.total_lbl]['mask']
    ixo = pmask == 0
    delta_state[ixo] = np.nan
    cmap = copy.copy(mymap)
    cmap.set_bad('white',1.)   
         
    sns.set_style('darkgrid')
    sns.set_context("notebook")
   
    plt.close(6) 
    fig,(ax,ax1) = plt.subplots(num=6, figsize=snow.figsize, 
                                dpi=snow.dpi, nrows = 1, ncols = 2)
    h = ax.imshow(delta_state, interpolation='none', 
        cmap = cmap, norm=MidpointNormalize(midpoint=0,
                                            vmin = vMin-0.01,vmax=vMax+0.01))

    if snow.basin == 'LAKES':    
        ax.set_xlim(snow.imgx)
        ax.set_ylim(snow.imgy)

    # Basin boundaries
    for name in snow.masks:
        ax.contour(snow.masks[name]['mask'],cmap = "Greys",linewidths = 1)

    if snow.basin == 'SJ':
        fix1 = np.arange(1275,1377)
        fix2 = np.arange(1555,1618)
        ax.plot(fix1*0,fix1,'k')
        ax.plot(fix2*0,fix2,'k')    
    
    # Do pretty stuff
    h.axes.get_xaxis().set_ticks([])
    h.axes.get_yaxis().set_ticks([])
    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="5%", pad=0.2)
    cbar = plt.colorbar(h, cax = cax)     
    # cbar.ax.tick_params() 
    
    if snow.units == 'KAF': 
        cbar.set_label(r'$\Delta$ SWE [in]')           
    if snow.units == 'SI':
        cbar.set_label(r'$\Delta$ SWE [mm]')   
        
    h.axes.set_title('Change in SWE \n %s to %s'
                     %(snow.dateFrom.date().strftime("%Y-%-m-%-d"),
                       snow.dateTo.date().strftime("%Y-%-m-%-d")))    
  
    # Plot the bar in order
    sumorder  = snow.plotorder[1:]  
    if snow.basin == 'LAKES' or snow.basin == 'RCEW':
        sumorder = [snow.plotorder[0]]  
    if snow.dplcs == 0:
        tlbl = '%s = %s %s'%(snow.plotorder[0],
                             str(int(snow.delta_state_byelev[snow.plotorder[0]].sum())),
                             snow.vollbl)
    else: 
        tlbl = '%s = %s %s'%(snow.plotorder[0],
                             str(np.round(snow.delta_state_byelev[snow.plotorder[0]].sum(),
                                          snow.dplcs)),snow.vollbl)            
    
    for iters,name in enumerate(sumorder):  
            
        if snow.dplcs == 0:
            lbl = '%s = %s %s'%(name,
                                str(int(snow.delta_state_byelev[name].sum())),
                                snow.vollbl)
        else: 
            lbl = '%s = %s %s'%(name,
                                str(np.round(snow.delta_state_byelev[name].sum(),
                                snow.dplcs)),snow.vollbl)                                 

        if iters == 0:
            ax1.bar(range(0,len(snow.edges)),snow.delta_state_byelev[name],
                    color = snow.barcolors[iters],
                    edgecolor = 'k',label = lbl)
        elif iters == 1:   
            ax1.bar(range(0,len(snow.edges)),snow.delta_state_byelev[name], 
                    bottom = snow.delta_state_byelev[sumorder[iters-1]], 
                    color = snow.barcolors[iters], edgecolor = 'k',label = lbl)
        elif iters == 2:   
            ax1.bar(range(0,len(snow.edges)),snow.delta_state_byelev[name], 
                    bottom = snow.delta_state_byelev[sumorder[iters-1]]
                    + snow.delta_state_byelev[sumorder[iters-2]], 
                    color = snow.barcolors[iters], edgecolor = 'k',label = lbl)
        elif iters == 3:   
            ax1.bar(range(0,len(snow.edges)),snow.delta_state_byelev[name], 
                    bottom = snow.delta_state_byelev[sumorder[iters-1]]
                    + snow.delta_state_byelev[sumorder[iters-2]]
                    + snow.delta_state_byelev[sumorder[iters-3]], 
                    color = snow.barcolors[iters], edgecolor = 'k',label = lbl)
           
    ax1.set_xlim((snow.xlims[0]-0.5,snow.xlims[1]))
    plt.tight_layout()                            
    xts = ax1.get_xticks()
    edges_lbl = []
    for i in xts[0:len(xts)-1]:
        edges_lbl.append(str(int(snow.edges[int(i)])))

    ax1.set_xticklabels(str(i) for i in edges_lbl)
    for tick in ax1.get_xticklabels():
        tick.set_rotation(30) 
         
    if hasattr(snow,"ch_ylims"):
        ax1.set_ylim(snow.ch_ylims)
    else:
        ylims = ax1.get_ylim()
        if ylims[0] < 0 and ylims[1] == 0:
            ax1.set_ylim((ylims[0]+(ylims[0]*0.3),ylims[1]+ylims[1]*0.3))
        if ylims[0] < 0 and ylims[1] > 0:
            ax1.set_ylim((ylims[0]+(ylims[0]*0.3),(ylims[1] + ylims[1]*0.9)))  
            if (ylims[1] + ylims[1]*0.9) < abs(ylims[0]):
                ax1.set_ylim((ylims[0]+(ylims[0]*0.3),(-(ylims[0]*0.6))))
                    
        if ylims[1] == 0:
            # ax1.set_ylim((ylims[0]+(ylims[0]*0.3),(-ylims[0])*0.5))
            ax1.set_ylim((ylims[0]+(ylims[0]*0.3),(-ylims[0])*0.65))
        if ylims[0] == 0:
            ax1.set_ylim((ylims[0]+(ylims[0]*0.3),ylims[1]+ylims[1]*0.3))               
                     
    if snow.units == 'KAF':
        ax1.set_ylabel('KAF - per elevation band')
        ax1.set_xlabel('elevation [ft]')
        ax1.axes.set_title('Change in SWE')
    if snow.units == 'SI':
        ax1.set_ylabel(r'$km^3$')
        ax1.set_xlabel('elevation [m]')
        ax1.axes.set_title(r'Change in SWE')
           
    ax1.yaxis.set_label_position("right")
    ax1.tick_params(axis='x')
    ax1.tick_params(axis='y')
    ax1.yaxis.tick_right()
    
    patches = [mpatches.Patch(color='grey', label='snow free')]
    if snow.basin == 'SJ':
        ax.legend(handles=patches, bbox_to_anchor=(0.3, 0.05),
                  loc=2, borderaxespad=0. )
    elif snow.basin == 'RCEW':
        ax.legend(handles=patches, bbox_to_anchor=(-0.1, 0.05),
                  loc=2, borderaxespad=0. )            
    else:
        ax.legend(handles=patches, bbox_to_anchor=(0.05, 0.05),
                  loc=2, borderaxespad=0. )
        
    if snow.basin != 'LAKES' and snow.basin != 'RCEW':
        # more ifs for number subs...
        if len(snow.plotorder) == 5:
            ax1.legend(loc= (0.01,0.68))
        elif len(snow.plotorder) == 4:
            ax1.legend(loc= (0.01,0.76))
        
    if snow.basin == 'BRB':
        ax1.text(0.26,0.96,tlbl,horizontalalignment='center',
                 transform=ax1.transAxes,fontsize = 10)
    
    if snow.basin == 'TUOL' or snow.basin == 'SJ':
        ax1.text(0.3,0.94,tlbl,horizontalalignment='center',
                 transform=ax1.transAxes,fontsize = 10)
  
    plt.tight_layout() 
    fig.subplots_adjust(top=0.88)
    
    print('saving figure to %sswe_change%s.png'%(snow.figs_path,snow.name_append))
    plt.savefig('%sswe_change%s.png'%(snow.figs_path,snow.name_append))    
    